//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw;

import java.io.File;
import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.frankmuenster.jameica.orverw.db.server.OrVerwDBServiceImpl;
import de.frankmuenster.jameica.orverw.gui.navigation.OrdnerVerwaltungExtension;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.gui.extension.ExtensionRegistry;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.plugin.Version;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Hauptklasse des Ordnerverwatungs-Plugins
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class OrdnerVerwaltung extends AbstractPlugin {

	/**
	 * Standart Konstruktor
	 */
	public OrdnerVerwaltung() { // NOPMD
		super();
		// intentionally left blank
	}

	/**
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#init()
	 */
	@Override
	public void init() throws ApplicationException {

		super.init();

		Logger.info("starting init process for OrdnerVerwaltung");

		// Datenbank Konsistenz berprfen
		call(new ServiceCall() {
			/**
			 * Ein Service call
			 */
			public void call(final OrVerwDBService service)
					throws ApplicationException, RemoteException {
				service.checkConsistency();
			}
		});

		// Menu des Jameica-'Plugins' erweitern
		ExtensionRegistry.register(new OrdnerVerwaltungExtension(),
				"jameica.start");

		Logger.info("ending init process for OrdnerVerwaltung");
	}

	/**
	 * This method is called only the first time, the plugin is loaded (before
	 * executing init()). if your installation procedure was not successfull,
	 * throw an ApplicationException. You dont need to implement this function.
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#install()
	 */
	@Override
	public void install() throws ApplicationException {

		// If we are running in client/server mode and this instance
		// is the client, we do not need to create a database.
		// Instead of this we will get our objects via RMI from
		// the server
		if (Application.inClientMode()) {
			return;
		}

		try {

			// Let's create an embedded Database
			final PluginResources res = Application.getPluginLoader()
					.getPlugin(OrdnerVerwaltung.class).getResources();
			final Manifest manifest = Application.getPluginLoader()
					.getManifest(OrdnerVerwaltung.class);
			final EmbeddedDatabase embeddedDb = new EmbeddedDatabase(
					res.getWorkPath() + "/db", "orverw", "orverwview");

			// create the sql tables.
			embeddedDb.executeSQLScript(new File(manifest.getPluginDir()
					+ File.separator + "sql", "create.sql"));

			// That's all. Database installed and tables created ;)
		} catch (Exception e) { // NOPMD
			Logger.error("Fehler : " + e.getMessage(), e.getCause());
			throw new ApplicationException("error while installing plugin", e);
		}
	}

	/**
	 * This method will be executed on every version change. You dont need to
	 * implement this function.
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#update(de.willuhn.jameica.plugin.Version)
	 */
	@Override
	@SuppressWarnings("PMD.UselessOverridingMethod")
	public void update(final Version oldVersion) throws ApplicationException { // 
		super.update(oldVersion);
	}

	/**
	 * Here you can do some cleanup stuff. The method will be called on every
	 * clean shutdown of jameica. You dont need to implement this function.
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#shutDown()
	 */

	public void shutDown() { // NOPMD
		super.shutDown();
	}

	/**
	 * Hilfsmethode zum bequemen Ausfuehren von Aufrufen auf dem Service.
	 */
	private interface ServiceCall {
		/**
		 * @param service
		 * @throws ApplicationException
		 * @throws RemoteException
		 */
		void call(OrVerwDBService service) throws ApplicationException,
				RemoteException;
	}

	/**
	 * Hilfsmethode zum bequemen Ausfuehren von Methoden auf dem Service.
	 * 
	 * @param call
	 *            der Call.
	 * @throws ApplicationException
	 */
	private void call(final ServiceCall call) throws ApplicationException {
		if (Application.inClientMode()) {
			return; // als Client muessen wir die DB nicht installieren
		}

		OrVerwDBService service = null; // NOPMD
		try {
			// Da die Service-Factory zu diesem Zeitpunkt noch nicht da ist,
			// erzeugen
			// wir uns eine lokale Instanz des Services.
			service = new OrVerwDBServiceImpl();
			service.start();
			call.call(service);
		} catch (ApplicationException ae) { // NOPMD
			throw ae;
		} catch (RemoteException e) {
			Logger.error("unable to init db service", e);
			final I18N i18n = getResources().getI18N();
			final StringBuilder msg = new StringBuilder(536);
			msg.append(
					i18n.tr("OrVerw-Datenbank konnte nicht initialisiert werden.\n\n{0} ",
							e.getMessage()))
					.append(msg
							.append("\n\nMglicherweise ist die Hibiscus-Datenbank defekt. Klicken Sie bitte auf \"Datei>Backups verwalten\", "
									+ "whlen Sie das Backup vom letzten Tag aus, an dem der Fehler noch nicht auftrat und klicken "
									+ "Sie anschlieend auf \"Ausgewhltes Backup wiederherstellen...\". Beim nchsten Start von Hibiscus "
									+ "wird das Backup automatisch wiederhergestellt. Sollte sich das Problem hierdurch nicht beheben lassen, "
									+ "besuchen Sie bitte http://www.willuhn.de/wiki/doku.php?id=support:fehlermelden"));

			throw new ApplicationException(msg.toString(), e);
		} finally {
			if (service != null) {
				try {
					service.stop(true);
				} catch (RemoteException e) {
					Logger.error("error while closing db service", e);
				}
			}
		}
	}

}
