//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;

import de.frankmuenster.jameica.orverw.gui.input.TextStyle;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.I18N;

/**
 * Settings Klasse fr den jameica.OrdnerVerwaltung Plugin
 * 
 * @author munstfr
 * @version 1.0
 * 
 * 
 */
public final class Settings {

	/**
	 * System settings von Jameica
	 */
	private static de.willuhn.jameica.system.Settings jameicaSettings = Application
			.getPluginLoader().getPlugin(OrdnerVerwaltung.class).getResources()
			.getSettings();

	/**
	 * Der DB Service
	 */
	private static DBService dbService;

	/**
	 * Der &Uuml;bersetzer
	 */
	private static I18N i18n = Application.getPluginLoader()
			.getPlugin(OrdnerVerwaltung.class).getResources().getI18N();

	// /**
	// * Our DateFormatter.
	// */
	// public final static DateFormat DATEFORMAT = DateFormat.getDateInstance(
	// DateFormat.DEFAULT, Application.getConfig().getLocale());
	//
	// /**
	// * Our decimal formatter.
	// */
	// public final static DecimalFormat DECIMALFORMAT = (DecimalFormat)
	// DecimalFormat
	// .getInstance(Application.getConfig().getLocale());
	//
	// /**
	// * Our currency name.
	// */
	// public final static String CURRENCY = "EUR";

	/**
	 * Privater Konstruktor, um ein Klassenerstellung zu verhindern
	 */
	private Settings() {
		// Intentionally left blank
	}

	/**
	 * Small helper function to get the database service.
	 * 
	 * @return db service.
	 * @throws RemoteException
	 */
	public static DBService getDBService() throws RemoteException {

		synchronized (DBService.class) {
			if (dbService == null) { 
				try {
					// We have to ask Jameica's ServiceFactory.
					// If we are running in Client/Server mode and we are the
					// client, the factory returns the remote dbService from the
					// Jameica server.
					// The name and class of the service is defined in
					// plugin.xml
					dbService = (DBService) Application.getServiceFactory()
							.lookup(OrdnerVerwaltung.class, "orVerwDB");
				} catch (Exception e) { // NOPMD
					throw new RemoteException(
							"error while getting database service", e);
				}
			}
		}
		return dbService;

	}

	/**
	 * Small helper function to get the translator.
	 * 
	 * @return translator.
	 */
	public static I18N getI18n() {
		// if (i18n == null)
		// i18n = Application.getPluginLoader()
		// .getPlugin(OrdnerVerwaltung.class).getResources().getI18N();

		return i18n;

	}

	/**
	 * Speichert die default Linienfarbe
	 * 
	 * @param color
	 */
	public static void setDefaultBarColor(final Color color) {
		jameicaSettings.setAttribute("orverw.barColor", color.getRGB());
	}

	/**
	 * Liefert die default Linienfarbe
	 * 
	 * @return die default Linienfarbe
	 */
	public static Color getDefaultBarColor() {
		final Color color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.barColor", new RGB(0, 0, 240)));
		return color;
	}

	/**
	 * Setzt die Sichtbarkeit der Linien
	 * 
	 * @param visible
	 */
	public static void setDefaultBarVisible(final Boolean visible) {
		jameicaSettings.setAttribute("orverw.barVisible", visible);
	}

	/**
	 * Liefert die default Sichtbarkeit der Linien
	 * 
	 * @return default Sichtbarkeit der Linien
	 */
	public static boolean isDefaultBarVisible() {
		return jameicaSettings.getBoolean("orverw.barVisible", true);
	}

	/**
	 * Setzt den default Texttype f&uuml;r den Inhalt
	 * 
	 * @param style
	 */
	public static void setDefaultContentStyle(final TextStyle style) {
		jameicaSettings.setAttribute("orverw.contentAlignment",
				style.getAlignment());
		jameicaSettings.setAttribute("orverw.contentColor", style
				.getTextColor().getRGB());
		jameicaSettings.setAttribute("orverw.contentFont", style.getFont()
				.getFontData()[0]);
	}

	/**
	 * Liefert den default Texttype f&uuml;r den Inhalt
	 * 
	 * @return default Texttype f&uuml;r den Inhalt
	 */
	public static TextStyle getDefaultContentStyle() {
		final TextStyle style = new TextStyle();
		style.setAlignment(jameicaSettings.getInt("orverw.contentAlignment",
				SWT.LEFT));
		final Color color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.contentColor", new RGB(0, 0, 0)));
		style.setTextColor(color);
		final Font font = new Font(GUI.getDisplay(),
				jameicaSettings.getFontData("orverw.contentFont",
						new FontData()));
		style.setFont(font);
		return style;
	}

	/**
	 * Setzt den default Texttype f&uuml;r den Fubereich
	 * 
	 * @param style
	 */
	public static void setDefaultFooterStyle(final TextStyle style) {
		jameicaSettings.setAttribute("orverw.footerAlignment",
				style.getAlignment());
		jameicaSettings.setAttribute("orverw.footerColor", style.getTextColor()
				.getRGB());
		jameicaSettings.setAttribute("orverw.footertFont", style.getFont()
				.getFontData()[0]);
	}

	/**
	 * Liefert den default Texttype f&uuml;r den Fubereich
	 * 
	 * @return default Texttype f&uuml;r den Fubereich
	 */
	public static TextStyle getDefaultFooterStyle() {
		final TextStyle style = new TextStyle();
		style.setAlignment(jameicaSettings.getInt("orverw.footerAlignment",
				SWT.LEFT));
		final Color color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.footerColor", new RGB(0, 0, 0)));
		style.setTextColor(color);
		final Font font = new Font(GUI.getDisplay(),
				jameicaSettings.getFontData("orverw.contentFont",
						new FontData()));
		style.setFont(font);
		return style;
	}

	/**
	 * Setzt die default Sichtbarkeit der ID
	 * 
	 * @param visible
	 */
	public static void setDefaultIdVisible(final Boolean visible) {
		jameicaSettings.setAttribute("orverw.idVisible", visible);
	}

	/**
	 * Liefert die default Sichtbarkeit der ID
	 * 
	 * @return default Sichtbarkeit der ID
	 */
	public static boolean isDefaultIdVisible() {
		return jameicaSettings.getBoolean("orverw.idVisible", true);
	}

	/**
	 * Setzt die default Label Hintergrundfarbe
	 * 
	 * @param color
	 */
	public static void setDefaultLabelBackground(final Color color) {
		jameicaSettings.setAttribute("orverw.labelBackground", color.getRGB());
	}

	/**
	 * Liefert die default Label Hintergrundfarbe
	 * 
	 * @return default Label Hintergrundfarbe
	 */
	public static Color getDefaultLabelBackground() {
		final Color color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.labelBackground", new RGB(250, 250, 190)));
		return color;
	}

	/**
	 * Setzt den default Texttype f&uuml;r die Register Beschreibung
	 * 
	 * @param style
	 */
	public static void setDefaultRegisterContentStyle(final TextStyle style) {
		jameicaSettings.setAttribute("orverw.registerContentBackground", style
				.getBackgroundColor().getRGB());
		jameicaSettings.setAttribute("orverw.registerContentColor", style
				.getTextColor().getRGB());
		jameicaSettings.setAttribute("orverw.registerContentFont", style
				.getFont().getFontData()[0]);
	}

	/**
	 * Liefert den default Texttype f&uuml;r die Register Beschreibung
	 * 
	 * @return default Texttype f&uuml;r die Register Beschreibung
	 */
	public static TextStyle getDefaultRegisterContentStyle() {
		final TextStyle style = new TextStyle();
		Color color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.registerContentBackground", new RGB(230, 230, 230)));
		style.setBackgroundColor(color);
		color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.registerContentColor", new RGB(0, 0, 0)));
		style.setTextColor(color);
		final Font font = new Font(GUI.getDisplay(),
				jameicaSettings.getFontData("orverw.registerContentFont",
						new FontData()));
		style.setFont(font);
		return style;
	}

	/**
	 * Setzt den default Texttype f&uuml;r den Register Index
	 * 
	 * @param style
	 */
	public static void setDefaultRegisterIndexStyle(final TextStyle style) {
		jameicaSettings.setAttribute("orverw.registerIndexBackground", style
				.getBackgroundColor().getRGB());
		jameicaSettings.setAttribute("orverw.registerIndexColor", style
				.getTextColor().getRGB());
		jameicaSettings.setAttribute("orverw.registerIndexFont", style
				.getFont().getFontData()[0]);
	}

	/**
	 * Liefert den default Texttype f&uuml;r den Register Index
	 * 
	 * @return default Texttype f&uuml;r den Register Index
	 */
	public static TextStyle getDefaultRegisterIndexStyle() {
		final TextStyle style = new TextStyle();
		Color color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.registerIndexBackground", new RGB(230, 230, 230)));
		style.setBackgroundColor(color);
		color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.registerIndexColor", new RGB(0, 0, 0)));
		style.setTextColor(color);
		final Font font = new Font(GUI.getDisplay(),
				jameicaSettings.getFontData("orverw.registerIndexFont",
						new FontData()));
		style.setFont(font);
		return style;
	}

	/**
	 * Setzt den default Texttype f&uuml;r den Label Titelbereich
	 * 
	 * @param style
	 */
	public static void setDefaultTitleStyle(final TextStyle style) {
		jameicaSettings.setAttribute("orverw.titleAlignment",
				style.getAlignment());
		jameicaSettings.setAttribute("orverw.titleColor", style.getTextColor()
				.getRGB());
		jameicaSettings.setAttribute("orverw.titleFont", style.getFont()
				.getFontData()[0]);
	}

	/**
	 * Liefert den default Texttype f&uuml;r den Label Titelbereich
	 * 
	 * @return default Texttype f&uuml;r den Label Titelbereich
	 */
	public static TextStyle getDefaultTitleStyle() {
		final TextStyle style = new TextStyle();
		style.setAlignment(jameicaSettings.getInt("orverw.titleAlignment",
				SWT.LEFT));
		final Color color = new Color(GUI.getDisplay(), jameicaSettings.getRGB(
				"orverw.titleColor", new RGB(0, 0, 0)));
		style.setTextColor(color);
		final Font font = new Font(GUI.getDisplay(),
				jameicaSettings.getFontData("orverw.titleFont", new FontData()));
		style.setFont(font);
		return style;
	}

}
