//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.orverw.OrdnerVerwaltung;
import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.willuhn.datasource.db.EmbeddedDBServiceImpl;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.messaging.QueryMessage;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.sql.version.Updater;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung unseres Datenbank Services
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class OrVerwDBServiceImpl extends EmbeddedDBServiceImpl implements
		OrVerwDBService {

	/**
	 * serialVersionUID
	 */
	private static final long serialVersionUID = 8818092640375364197L;

	/**
	 * @throws RemoteException
	 *             rmi-exception
	 */
	public OrVerwDBServiceImpl() throws RemoteException {
		super(Application.getPluginLoader().getPlugin(OrdnerVerwaltung.class)
				.getResources().getWorkPath()
				+ "/db/db.conf", "orverw", "orverwview");

		// We have to define jameicas classfinder.
		// otherwise, the db service will not be able to find
		// implementors by their interfaces.
		this.setClassFinder(Application.getClassLoader().getClassFinder());
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService#install()
	 */
	@Override
	public void install() throws RemoteException {

		// Let's create an embedded Database
		final PluginResources res = Application.getPluginLoader()
				.getPlugin(OrdnerVerwaltung.class).getResources();
		final Manifest manifest = Application.getPluginLoader().getManifest(
				OrdnerVerwaltung.class);
		EmbeddedDatabase embeddedDb;

		try {
			embeddedDb = new EmbeddedDatabase(res.getWorkPath() + "/db",
					"orverw", "orverwview");

			// create the sql tables.
			embeddedDb.executeSQLScript(new File(manifest.getPluginDir()
					+ File.separator + "sql", "create.sql"));

		} catch (Exception e) { // NOPMD 
			throw new RemoteException(e.getMessage(), e);
		}

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService#checkConsistency()
	 */
	@Override
	public void checkConsistency() throws RemoteException, ApplicationException {
		Logger.info("init update provider");
		final UpdateProvider provider = new OrVerwUpdateProvider(
				getConnection(), VersionUtility.getVersion(this, "db"));
		final Updater updater = new Updater(provider, "iso-8859-1");
		updater.execute();
		Logger.info("updates finished");

	}

	/**
	 * @see de.willuhn.datasource.db.DBServiceImpl#getConnection()
	 */
	protected Connection getConnection() throws RemoteException {
		try {
			return super.getConnection();
		} catch (RemoteException re) {
			// Wir benachrichtigen Jameica ueber den Fehler, damit beim Shutdown
			// kein Backup erstellt wird
			Application.getMessagingFactory()
					.getMessagingQueue("jameica.error")
					.sendMessage(new QueryMessage(re));
			throw re;
		}
	}

}
