//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.orverw.OrdnerVerwaltung;
import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Version;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Der UpdateProvider der sich um Dantenbank und ander Updates k&uuml;mmert.
 * 
 * @author munstfr
 * @version 1.0
 */
public class OrVerwUpdateProvider implements UpdateProvider {

	/**
	 * Die Version
	 */
	private transient final Version version;
	/**
	 * Ein SQL Verbindungsobjekt
	 */
	private transient final Connection conn;
	/**
	 * Das Plugin-Manifest
	 */
	private transient final Manifest manifest;
	/**
	 * Die Plugin Resoucen
	 */
	private transient final PluginResources res;

	/**
	 * Konstruktor mit Verbindungsobjekt und Versionstablle
	 * 
	 * @param conn
	 *            SQL Verbindungsobjekt
	 * @param version
	 *            Versionstabelle
	 */
	protected OrVerwUpdateProvider(final Connection conn, final Version version) {
		this.conn = conn;
		this.version = version;

		final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(
				OrdnerVerwaltung.class);
		this.manifest = plugin.getManifest();
		this.res = plugin.getResources();
	}

	/**
	 * @see de.willuhn.sql.version.UpdateProvider#getConnection()
	 */
	@Override
	public Connection getConnection() throws ApplicationException {
		return this.conn;
	}

	/**
	 * @return die PLuginResource
	 */
	public PluginResources getResources() {
		return this.res;
	}

	/**
	 * @see de.willuhn.sql.version.UpdateProvider#getCurrentVersion()
	 */
	@Override
	public int getCurrentVersion() throws ApplicationException {
		try {
			return this.version.getVersion();
		} catch (RemoteException re) {
			Logger.error("Error reading current version number");
			throw new ApplicationException(Settings.getI18n().tr(
					"Error reading current version number"), re);
		}
	}

	/**
	 * @see de.willuhn.sql.version.UpdateProvider#getProgressMonitor()
	 */
	@Override
	public ProgressMonitor getProgressMonitor() {
		// Liefert den Splashscreen oder im Servermode einen
		// Pseudo-Monitor.
		return Application.getController().getApplicationCallback()
				.getStartupMonitor();
	}

	/**
	 * @see de.willuhn.sql.version.UpdateProvider#getUpdatePath()
	 */
	@Override
	public File getUpdatePath() throws ApplicationException {
		// Ist das Unterverzeichnis "plugins" im Plugin
		return new File(manifest.getPluginDir(), "updates");
	}

	/**
	 * @see de.willuhn.sql.version.UpdateProvider#setNewVersion(int)
	 */
	@Override
	public void setNewVersion(final int newVersion) throws ApplicationException {
		final int current = getCurrentVersion();
		try {
			if (Logger.isLogging(Level.INFO)) {
				Logger.info("applying new version [" + this.version.getName() // NOPMD 
						+ "]: " + newVersion);
			}
			this.version.setVersion(newVersion);
			this.version.store();
		} catch (RemoteException e) {
			// Im Fehlerfall Versionsnummer zuruecksetzen
			try {
				this.version.setVersion(current);
			} catch (RemoteException e2) {
				Logger.error("unable to rollback version", e2);
				// Werfen wir nicht, weil es sonst die eigentliche Exception
				// verdecken wuerde
			}

			Logger.error("unable to read current version number", e);
			throw new ApplicationException(Settings.getI18n().tr(
					"Fehler beim Ermitteln der aktuellen Versionsnummer"), e);
		} catch (ApplicationException e) {
			// Im Fehlerfall Versionsnummer zuruecksetzen
			try {
				this.version.setVersion(current);
			} catch (RemoteException e2) {
				Logger.error("unable to rollback version", e2);
				// Werfen wir nicht, weil es sonst die eigentliche Exception
				// verdecken wuerde
			}
			throw (ApplicationException) e;
		}

	}

}
