/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.orverw.db.server; 

import java.rmi.RemoteException;
import java.util.Date;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.gui.input.TextStyle;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung f&uuml; die Ordnertabelle
 * 
 * <pre>
 * CREATE TABLE ordner (
 *   id NUMERIC default UNIQUEKEY('ordner'),
 *   title varchar(80) NOT NULL,
 *   titleFont varchar(80),
 *   titleColor int,
 *   titleAlignmnet int,
 *   content varchar(300),
 *   contentFont varchar(80),
 *   contentColor int,
 *   contentAlignmnet int,
 *   footer varchar(100),
 *   footerFont varchar(80),
 *   footerColor int,
 *   footerAlignmnet int,
 *   labelBackground int,
 *   horizontalBarVisible boolean default true,
 *   horizontalBarColor int,
 *   idVisible boolean default true,
 *   registerIndexFont varchar(80),
 *   registerIndexColor int,
 *   registerIndexBackground int,
 *   registerContentFont varchar(80),
 *   registerContentColor int,
 *   registerContentBackground int,
 *   width int,
 *   lastSaveDate date,
 *   UNIQUE (id),
 *   PRIMARY KEY (id)
 * );
 * </pre>
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings({"PMD.GodClass","PMD.ExcessivePublicCount"})
public class OrdnerImpl extends AbstractDBObject implements Ordner {

	/**
	 * Die korrete Anzahl an Font Attributen, die vorhanden sein m&uuml;ssen
	 */
	private static final int CORRECT_FONT_ATTRIBUTE_LENNGTH = 4; // NOPMD

	/**
	 * Standart Konstruktor
	 * 
	 * @throws RemoteException
	 */
	public OrdnerImpl() throws RemoteException {
		super();
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getBarColor()
	 */
	@Override
	public Color getBarColor() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("horizontalBarColor");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getContent()
	 */
	@Override
	public String getContent() throws RemoteException {
		return (String) getAttribute("content");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getContentAlignment()
	 */
	@Override
	public int getContentAlignment() throws RemoteException {
		return (Integer) getAttribute("contentAlignmnet");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getContentColor()
	 */
	@Override
	public Color getContentColor() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("contentColor");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getContentFont()
	 */
	@Override
	public Font getContentFont() throws RemoteException {
		final String fontAsString = (String) getAttribute("contentFont");
		return convertStringToFont(fontAsString);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getDate()
	 */
	@Override
	public Date getDate() throws RemoteException {
		return (Date) getAttribute("lastSaveDate");
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getFooter()
	 */
	@Override
	public String getFooter() throws RemoteException {
		return (String) getAttribute("footer");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getFooterAlignment()
	 */
	@Override
	public int getFooterAlignment() throws RemoteException {
		return (Integer) getAttribute("footerAlignmnet");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getFooterColor()
	 */
	@Override
	public Color getFooterColor() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("footerColor");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getFooterFont()
	 */
	@Override
	public Font getFooterFont() throws RemoteException {
		final String fontAsString = (String) getAttribute("footerFont");
		return convertStringToFont(fontAsString);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getInhaltsverzeichniss()
	 */
	@Override
	public DBIterator getInhaltsverzeichniss() throws RemoteException {
		try {
			// 1) Get the Database Service.
			final DBService service = this.getService();

			// 3) We create the history value list using getList(Class)
			final DBIterator register = service.createList(Register.class);

			// 4) we add a filter to only query for tasks with our project id
			register.addFilter("ordner_id = " + this.getID());
			register.setOrder("ORDER BY order_id asc");

			return register;
		} catch (RemoteException e) {
			throw new RemoteException("unable to load order content list", e);
		}
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getLabelBackground()
	 */
	@Override
	public Color getLabelBackground() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("labelBackground");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return "id";
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getRegisterContentBackground()
	 */
	@Override
	public Color getRegisterContentBackground() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("registerContentBackground");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getRegisterContentColor()
	 */
	@Override
	public Color getRegisterContentColor() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("registerContentColor");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getRegisterContentFont()
	 */
	@Override
	public Font getRegisterContentFont() throws RemoteException {
		final String fontAsString = (String) getAttribute("registerContentFont");
		return convertStringToFont(fontAsString);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getRegisterIndexBackground()
	 */
	@Override
	public Color getRegisterIndexBackground() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("registerIndexBackground");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getRegisterIndexColor()
	 */
	@Override
	public Color getRegisterIndexColor() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("registerIndexColor");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getRegisterIndexFont()
	 */
	@Override
	public Font getRegisterIndexFont() throws RemoteException {
		final String fontAsString = (String) getAttribute("registerIndexFont");
		return convertStringToFont(fontAsString);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getTitel()
	 */
	@Override
	public String getTitel() throws RemoteException {
		return (String) getAttribute("title");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getTitelAlignment()
	 */
	@Override
	public int getTitelAlignment() throws RemoteException {
		return (Integer) getAttribute("titleAlignmnet");

	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getTiteColor()
	 */
	@Override
	public Color getTitelColor() throws RemoteException {
		final Integer rgb = (Integer) getAttribute("titleColor");
		return convertRgbIntToColor(rgb);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getTitelFont()
	 */
	@Override
	public Font getTitelFont() throws RemoteException {
		final String fontAsString = (String) getAttribute("titleFont");
		return convertStringToFont(fontAsString);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getSize()
	 */
	@Override
	@SuppressWarnings("PMD.NullAssignment")
	public Integer getWidth() throws RemoteException {

		final Integer width = (Integer) getAttribute("width");
		return (width == null) ? null : width; 
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#isBarVisible()
	 */
	@Override
	public boolean isBarVisible() throws RemoteException {
		return (Boolean) getAttribute("horizontalBarVisible");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#isIdVisible()
	 */
	@Override
	public boolean isIdVisible() throws RemoteException {
		return (Boolean) getAttribute("idVisible");
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#loadDefaults()
	 */
	@Override
	public void loadDefaults() throws RemoteException {

		this.setBarColor(Settings.getDefaultBarColor());
		this.setBarVisible(Settings.isDefaultBarVisible());
		TextStyle style = Settings.getDefaultContentStyle();
		this.setContentAlignment(style.getAlignment());
		this.setContentColor(style.getTextColor());
		this.setContentFont(style.getFont());
		style = Settings.getDefaultFooterStyle();
		this.setFooterAlignment(style.getAlignment());
		this.setFooterColor(style.getTextColor());
		this.setFooterFont(style.getFont());
		this.setIdVisible(Settings.isDefaultIdVisible());
		this.setLabelBackground(Settings.getDefaultLabelBackground());
		style = Settings.getDefaultRegisterContentStyle();
		this.setRegisterContentBackground(style.getBackgroundColor());
		this.setRegisterContentColor(style.getTextColor());
		this.setRegisterContentFont(style.getFont());
		style = Settings.getDefaultRegisterIndexStyle();
		this.setRegisterIndexBackground(style.getBackgroundColor());
		this.setRegisterIndexColor(style.getTextColor());
		this.setRegisterIndexFont(style.getFont());
		style = Settings.getDefaultTitleStyle();
		this.setTitelAlignment(style.getAlignment());
		this.setTitelColor(style.getTextColor());
		this.setTitelFont(style.getFont());

		// setzte ein paar Dummy Texte
		this.setContent("Thema1\nThema2");
		this.setFooter("Ordner 1/1");
		this.setTitel("Titel");

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setBarColor(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setBarColor(final Color color) throws RemoteException {
		setAttribute("horizontalBarColor", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setBarVisible(boolean)
	 */
	@Override
	public void setBarVisible(final boolean visible) throws RemoteException {
		setAttribute("horizontalBarVisible", visible);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#setContent(java.lang.String)
	 */
	@Override
	public void setContent(final String inhalt) throws RemoteException {
		setAttribute("content", inhalt);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setContentAlignment(int)
	 */
	@Override
	public void setContentAlignment(final int alignment) throws RemoteException {
		setAttribute("contentAlignmnet", alignment);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setContentColor(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setContentColor(final Color color) throws RemoteException {
		setAttribute("contentColor", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setContentFont(org.eclipse.swt.graphics.Font)
	 */
	@Override
	public void setContentFont(final Font font) throws RemoteException {
		setAttribute("contentFont", convertFontToString(font));
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#setDate(java.util.Date)
	 */
	@Override
	public void setDate(final Date datum) throws RemoteException {
		setAttribute("lastSaveDate", datum);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#setFooter(java.lang.String)
	 */
	@Override
	public void setFooter(final String fussnote) throws RemoteException {
		setAttribute("footer", fussnote);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setFooterAlignment(int)
	 */
	@Override
	public void setFooterAlignment(final int alignment) throws RemoteException {
		setAttribute("footerAlignmnet", alignment);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setFooterColor(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setFooterColor(final Color color) throws RemoteException {
		setAttribute("footerColor", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setFooterFont(org.eclipse.swt.graphics.Font)
	 */
	@Override
	public void setFooterFont(final Font font) throws RemoteException {
		setAttribute("footerFont", convertFontToString(font));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setIdVisible(boolean)
	 */
	@Override
	public void setIdVisible(final boolean visible) throws RemoteException {
		setAttribute("idVisible", visible);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setLabelBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setLabelBackground(final Color color) throws RemoteException {
		setAttribute("labelBackground", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setRegisterContentBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setRegisterContentBackground(final Color color)
			throws RemoteException {
		setAttribute("registerContentBackground", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setRegisterContentColor(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setRegisterContentColor(final Color color)
			throws RemoteException {
		setAttribute("registerContentColor", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setRegisterContentFont(org.eclipse.swt.graphics.Font)
	 */
	@Override
	public void setRegisterContentFont(final Font font) throws RemoteException {
		setAttribute("registerContentFont", convertFontToString(font));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setRegisterIndexBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setRegisterIndexBackground(final Color color)
			throws RemoteException {
		setAttribute("registerIndexBackground", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setRegisterIndexColor(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setRegisterIndexColor(final Color color) throws RemoteException {
		setAttribute("registerIndexColor", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setRegisterIndexFont(org.eclipse.swt.graphics.Font)
	 */
	@Override
	public void setRegisterIndexFont(final Font font) throws RemoteException {
		setAttribute("registerIndexFont", convertFontToString(font));
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#setTitel(java.lang.String)
	 */
	@Override
	public void setTitel(final String titel) throws RemoteException {
		setAttribute("title", titel);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setTitelAlignment(int)
	 */
	@Override
	public void setTitelAlignment(final int alignment) throws RemoteException {
		setAttribute("titleAlignmnet", alignment);
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#getTitelColor(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setTitelColor(final Color color) throws RemoteException {
		setAttribute("titleColor", convertColorToRgbInt(color));
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.db.rmi.Ordner#setTitelFont(org.eclipse.swt.graphics.Font)
	 */
	@Override
	public void setTitelFont(final Font font) throws RemoteException {
		setAttribute("titleFont", convertFontToString(font));
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#setWidth(java.lang.Integer)
	 */
	@Override
	public void setWidth(final Integer width) throws RemoteException {
		setAttribute("width", width);
	}

	private int convertColorToRgbInt(final Color color) {

		int rgb;
		if (color == null) {
			rgb = -1;
		} else {
			// create long aus RGB
			rgb = 65536 * color.getRed() + 256 * color.getGreen()
					+ color.getBlue();
		}
		return rgb;

	}

	private String convertFontToString(final Font font) {

		String converted;

		if (font == null) {
			converted = "";
		} else {
			final StringBuffer fontData = new StringBuffer();
			final FontData fontdata = font.getFontData()[0];

			fontData.append(fontdata.getHeight()).append(';')
					.append(fontdata.getLocale()).append(';')
					.append(fontdata.getName()).append(';')
					.append(fontdata.getStyle()).append(';');
			converted = fontData.toString();

		}

		return converted;

	}

	private Color convertRgbIntToColor(final Integer rgbInt) {

		Color color;
		if (rgbInt == null) {
			color = null; // NOPMD
		} else {

			final int red = (int) (rgbInt % 0x1000000 / 0x10000);
			final int green = (int) (rgbInt % 0x10000 / 0x100);
			final int blue = (int) (rgbInt % 0x100);

			color = new Color(GUI.getDisplay(), red, green, blue);
		}

		return color;
	}

	private Font convertStringToFont(final String fontAsString) {

		Font font;
		if (fontAsString == null) {
			font = GUI.getShell().getFont();
		} else {
			final String[] fontAttributtes = fontAsString.split(";");
			if (fontAttributtes.length == CORRECT_FONT_ATTRIBUTE_LENNGTH) {
				final FontData fontdata = new FontData();
				fontdata.setHeight(Integer.valueOf(fontAttributtes[0]));
				fontdata.setLocale(fontAttributtes[1]);
				fontdata.setName(fontAttributtes[2]);
				fontdata.setStyle(Integer.valueOf(fontAttributtes[3]));
				font = new Font(GUI.getDisplay(), fontdata);
			} else {
				font = GUI.getShell().getFont();
			}
		}

		return font;

	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
	 */
	protected void deleteCheck() throws ApplicationException {
		// nothing to do here
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
	 */
	@Override
	protected String getTableName() {
		return "ordner";
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
	 */
	protected void insertCheck() throws ApplicationException {
		try {
			if (getTitel() == null) {
				setTitel("");
			}
			if (getContent() == null) {
				setContent("");
			}
			if (getFooter() == null) {
				setFooter("");
			}
			if (getDate() == null) {
				setDate(new Date());
			}

		} catch (RemoteException e) {
			Logger.error("insert check of ordner failed", e);
			throw new ApplicationException(Settings.getI18n().tr(
					"unable to store ordner, please check the system log"), e);
		}
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
	 */
	protected void updateCheck() throws ApplicationException {
		// same as insertCheck
		insertCheck();
	}

}
