//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw.db.server;

import java.rmi.RemoteException;
import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.ObjectNotFoundException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * 
 * Die Implementierung des Datenbank Interface zu den Registereintr&auml;gen
 * 
 * <pre>
 *   id NUMERIC default UNIQUEKEY('inhaltsverzeichniss'), 
 *   ordner_id int(4) NOT NULL, 
 *   idx varchar(10), 
 *   content varchar(80), 
 *   orderId int, 
 *   UNIQUE (id), PIMARY KEY (id)
 * </pre>
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class RegisterImpl extends AbstractDBObject implements Register {

	/**
	 * serialVersionUID
	 */
	private static final long serialVersionUID = 6650563945934599214L;

	/**
	 * @throws RemoteException
	 */
	public RegisterImpl() throws RemoteException {
		super();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
	 */
	@Override
	protected void deleteCheck() throws ApplicationException {
		// hier brauchen haben wir nichts zu tun
	}

	/** 
	 * @see
	 * de.frankmuenster.jameica.orverw.db.rmi.Register#getBeschreibung()
	 */
	@Override
	public String getBeschreibung() throws RemoteException {
		return (String) getAttribute("content");
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
	 */
	@Override
	protected Class<Ordner> getForeignObject(final String field)
			throws RemoteException {
		// the system is able to resolve foreign keys and loads
		// the according objects automatically. You only have to
		// define which class handles which foreign key.
		if ("ordner_id".equals(field)) { // NOPMD 
			return Ordner.class; // NOPMD 
		}
		return null;
	}

	/** 
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Register#getIndex()
	 */
	@Override
	public String getIndex() throws RemoteException {
		return (String) getAttribute("idx");
	}

	/** 
	 * @see
	 * de.frankmuenster.jameica.orverw.db.rmi.Register#getOrdner()
	 */
	@Override
	public Ordner getOrdner() throws RemoteException {
		// Yes, we can cast this directly to Ordner, because
		// getForeignObject(String)
		// contains the mapping for this attribute.
		try {
			return (Ordner) getAttribute("ordner_id"); // NOPMD 
		} catch (ObjectNotFoundException e) {
			return null;
		}
	}

	/** 
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Register#getOrdnerId()
	 */
	@Override
	public int getOrdnerId() throws RemoteException {
		final Integer ordner_id = (Integer) getAttribute("ordner_id");
		if (ordner_id != null) {
			return ordner_id; // NOPMD 
		}
		return 0;
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return "id";
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
	 */
	@Override
	protected String getTableName() {
		return "register";
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
	 */
	@Override
	protected void insertCheck() throws ApplicationException {
		try {
			if (getIndex() == null) {
				setIndex("");
			}
			if (getBeschreibung() == null) {
				setBeschreibung("");
			}

			if (getOrdner() == null) {
				throw new ApplicationException(Settings.getI18n().tr(
						"Please first choose a folder"));
			}

			if (getOrdner().isNewObject()) {
				throw new ApplicationException(Settings.getI18n().tr(
						"Please first save folder"));
			}

		} catch (RemoteException e) {
			Logger.error(Settings.getI18n().tr("insert check of regsiter failed"), e);
			throw new ApplicationException(
					Settings.getI18n()
							.tr("unable to store register line, please check the system log"), e);
		}
	}

	/** 
	 * @see
	 * de.frankmuenster.jameica.orverw.db.rmi.Register#setBeschreibung(java.lang.String)
	 */
	@Override
	public void setBeschreibung(final String beschreibung) throws RemoteException {
		setAttribute("content", beschreibung);
	}

	/** 
	 * @see
	 * de.frankmuenster.jameica.orverw.db.rmi.Register#setIndex(java.lang.String)
	 */
	@Override
	public void setIndex(final String index) throws RemoteException {
		setAttribute("idx", index);
	}

	/** 
	 * @see
	 * de.frankmuenster.jameica.orverw.db.rmi.Register#setOrdner(
	 * de.frankmuenster.jameica.orverw.db.rmi.Ordner)
	 */
	@Override
	public void setOrdner(final Ordner ordner) throws RemoteException {
		// casting wird in foreignObject bernommen
		setAttribute("ordner_id", ordner);
	}

	/** 
	 * @see de.frankmuenster.jameica.orverw.db.rmi.Register#setOrdnerId(int)
	 */
	@Override
	public void setOrdnerId(final int orderId) throws RemoteException {
		setAttribute("ordner_id", orderId);

	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
	 */
	@Override
	protected void updateCheck() throws ApplicationException {
		// same as insertCheck
		insertCheck();
	}

}
