//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 
package de.frankmuenster.jameica.orverw.gui.action;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.ApplicationCallback;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Action Klasse zum L&ouml;schen von Ordnern
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class OrdnerDeleteAction implements Action {

	/**
	 * Unser &Uunl;bersetzer
	 */
	private final static I18N I18N = Settings.getI18n();
	/**
	 * Callback f&uunl; Benutzernachfragen
	 */
	private final static ApplicationCallback CALLBACK = Application
			.getCallback();

	/**
	 * Standart Konstruktor
	 */
	public OrdnerDeleteAction() { // NOPMD 
		// This constructor is intentionally empty. Nothing special is needed
		// here.
	}

	/**
	 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	 */
	@Override
	public void handleAction(Object context) throws ApplicationException { // NOPMD

		if (context == null || !(context instanceof Ordner)
				&& !(context instanceof Ordner[])) {
			throw new ApplicationException(
					I18N.tr("Please choose a folder(s) you want to delete"));
		}

		if (context instanceof Ordner) {

			handleDeleteOrdner((Ordner) context);

		} else {
			handleDeleteOrdner((Ordner[]) context);
		}

	}

	private void handleDeleteOrdner(final Ordner ordner)
			throws ApplicationException {

		// before deleting the folder, we show up a confirm dialog ;)
		final String question = I18N
				.tr("Do you really want to delete this folder?");
		try {
			if (CALLBACK.askUser(question)) {
				deleteOrdner(ordner);
				Application.getMessagingFactory().sendMessage(
						new StatusBarMessage(I18N
								.tr("Folder deleted successfully"),
								StatusBarMessage.TYPE_SUCCESS));
			}
		} catch (Exception e) { // NOPMD 
			Logger.error("error while deleting folder", e);
			throw new ApplicationException(
					I18N.tr("Error while deleting folder"), e);
		}
	}

	private void handleDeleteOrdner(final Ordner[] ordnerArray) // NOPMD
			throws ApplicationException {
		// before deleting the folder, we show up a confirm dialog ;)
		final String question = I18N
				.tr("Do you really want to delete this folders?");
		try {
			if (CALLBACK.askUser(question)) {
				for (final Ordner ordner : (Ordner[]) ordnerArray) {
					deleteOrdner(ordner);
				}
				Application.getMessagingFactory().sendMessage(
						new StatusBarMessage(I18N
								.tr("Folders deleted successfully"),
								StatusBarMessage.TYPE_SUCCESS));
			}
		} catch (Exception e) { // NOPMD
			Logger.error("error while deleting folders", e);
			throw new ApplicationException(
					I18N.tr("Error while deleting folders"), e);
		}
	}

	private void deleteOrdner(final Ordner ordner) throws RemoteException,
			ApplicationException {
		final DBIterator register = ordner.getInhaltsverzeichniss();
		while (register.hasNext()) {
			final Register registerEintrag = (Register) register.next();
			registerEintrag.delete();
		}
		ordner.delete();
	}
}
