/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */
package de.frankmuenster.jameica.orverw.gui.controller;

import java.io.File;
import java.io.FileInputStream;

import de.frankmuenster.jameica.orverw.OrdnerVerwaltung;
import de.frankmuenster.jameica.orverw.Settings;
import de.willuhn.io.FileFinder;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.util.InfoReader;
import de.willuhn.logging.Logger;
import de.willuhn.util.I18N;

/**
 * Controller f&uuml;r den Lizenzinformationen Dialog
 * 
 * Basiert auf dem
 * {@link de.willuhn.jameica.gui.internal.controller#LincenseControl}
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
@SuppressWarnings("PMD.BeanMembersShouldSerialize")
public class LicenseControl extends AbstractControl {

	/**
	 * I18N bersetzungs-Objekt aus unseren Settings
	 */
	private final static I18N I18N = Settings.getI18n();

	/**
	 * Zeilenvorschub
	 */
	public final static String LINE_BREAK = "<br/>";

	/**
	 * FormTextPart um den Text schn formatieren zu knnen
	 */
	private FormTextPart libList;

	/**
	 * Standart Konstruktor liefert ein LincenceControl object
	 * 
	 * @param view
	 *            die View, f&uuml;r die dieser LicenceControl zustndig ist
	 */
	public LicenseControl(final AbstractView view) {
		super(view);
	}

	/**
	 * Liefert eine Liste mit allen direkt von jameica.qrscanner verwendeten
	 * Komponenten.
	 * 
	 * @return Liste der Komponenten.
	 */
	@SuppressWarnings({ "PMD.AvoidInstantiatingObjectsInLoops",
			"PMD.AvoidCatchingGenericException" })
	public Part getLibList() {
		if (libList == null) {

			final AbstractPlugin plugin = Application.getPluginLoader()
					.getPlugin(OrdnerVerwaltung.class);

			final StringBuffer buffer = new StringBuffer();
			final Manifest manifest = plugin.getManifest();

			buffer.append("<form><p><span color=\"header\" font=\"header\">"
					+ I18N.tr("Folder Manager") + "</span></p>");
			if (manifest != null) {
				buffer.append("<p>");
				buffer.append(manifest.getDescription());
				buffer.append(LINE_BREAK + manifest.getHomepage());
				buffer.append(LINE_BREAK + manifest.getLicense());
				buffer.append("</p>");
			}

			buffer.append("<p><br/><span color=\"header\" font=\"header\">"
					+ I18N.tr("Used Components") + "</span></p>");
			final String path = plugin.getManifest().getPluginDir();

			final FileFinder finder = new FileFinder(new File(path + "/lib"));
			finder.matches(".*?info\\.xml$");
			final File[] infos = finder.findRecursive();
			for (int i = 0; i < infos.length; ++i) {
				if (!infos[i].isFile() || !infos[i].canRead()) {
					Logger.warn("unable to read " + infos[i] + ", skipping");
					continue;
				}

				try {
					final InfoReader infoReader = new InfoReader(
							new FileInputStream(infos[i]));
					buffer.append("<p><b>" + infoReader.getName() + "</b>");
					buffer.append(LINE_BREAK + infoReader.getDescription());
					buffer.append(LINE_BREAK + infoReader.getUrl());
					buffer.append(LINE_BREAK + infoReader.getLicense());
					buffer.append(" </p>");
				} catch (Exception e) { 
					Logger.error("unable to parse " + infos[0], e);
				}
			}
			buffer.append("</form>");

			libList = new FormTextPart(buffer.toString());
		}

		return libList;
	}
}
