/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.gui.controller;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.FontDialog;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.db.server.RegisterImpl;
import de.frankmuenster.jameica.orverw.gui.dialog.PrintDialog;
import de.frankmuenster.jameica.orverw.gui.input.ToolTipButton;
import de.frankmuenster.jameica.orverw.gui.menu.RegisterTableMenu;
import de.frankmuenster.jameica.orverw.gui.views.OrdnerListView;
import de.frankmuenster.jameica.orverw.parts.RegisterTablePart;
import de.frankmuenster.jameica.orverw.parts.RegisterTypEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPart;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartButtonObject;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartListener;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelSizeEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsButtonObject;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsButtonsEnum;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsListener;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsPart;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.TableChangeListener;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * @author munstfr
 * 
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.ExcessiveImports",
		"PMD.CyclomaticComplexity", "PMD.StdCyclomaticComplexity",
		"PMD.TooManyMethods", "PMD.DataflowAnomalyAnalysis" })
public class OrdnerBearbeitenController extends AbstractControl implements
		LabelPartListener, TableChangeListener, RegisterSettingsListener {

	/**
	 * Unser &Uuml;bersetzer
	 */
	private static final I18N I18N = Settings.getI18n();

	/**
	 * Unser Ordner
	 */
	private transient Ordner ordner;
	/**
	 * und die Registereintr&auml;ge
	 */
	private transient DBIterator register;

	/**
	 * Ist es ein neuer Ordner
	 */
	private transient boolean neuerOrdner;
	/**
	 * Die Gr&ouml;&szlig;e des Labels
	 */
	private transient LabelSizeEnum labelSize;
	/**
	 * Das Part f&uuml;r den Ordnerr&uuml;cken
	 */
	private transient LabelPart ordnerRuecken;
	/**
	 * Das Part f&uuml;r die Registereintr&auml;ge
	 */
	private transient RegisterTablePart registerTable;
	/**
	 * Das Part f&uuml; die Regsitereinstellungen
	 */
	private transient RegisterSettingsPart registerSettings;

	// Buttons
	/**
	 * Unser Speicher-Button
	 */
	private transient Button saveButton;
	/**
	 * Unser Abbrech-Button
	 */
	private transient Button cancelEditButton;
	/**
	 * Die Buttons f&uuml; die unterschiedlichen Label-Gr&ouml;&szlig;en
	 */
	private transient List<ToolTipButton> registerSizeBtns;
	/**
	 * Der Druck-Button
	 */
	private transient Button printButton;

	/**
	 * Zeigt an, ob sich der Inhalt ver&auml;ndert hat.
	 */
	private transient boolean inhaltChanged;

	/**
	 * Der Warntext f&uuml;r Rechtsklickproblem
	 */
	private transient String linuxWarning;

	/**
	 * Unser Konstruktor f&uuml;r den Knotroller
	 * 
	 * @param view
	 * @throws RemoteException
	 */
	public OrdnerBearbeitenController(final AbstractView view)
			throws RemoteException {
		super(view);

		final Object viewObject = view.getCurrentObject();

		if (viewObject instanceof LabelSizeEnum) {
			neuerOrdner = true;
			labelSize = (LabelSizeEnum) viewObject;

			final DBService service = Settings.getDBService();
			ordner = (Ordner) service.createObject(Ordner.class, null);
			ordner.setWidth(labelSize.getWidth());
			ordner.loadDefaults();

		} else if (viewObject instanceof Ordner) {
			neuerOrdner = false;
			ordner = (Ordner) getCurrentObject();
			register = ordner.getInhaltsverzeichniss();
			final Integer width = ordner.getWidth();
			labelSize = LabelSizeEnum.getLabelSize(width);

		}
		inhaltChanged = false;

	}

	/**
	 * @return <i>true</i> wenn es ein neuer Ordner ist, <i>false</i> sonst.
	 */
	public boolean isNeuerOrdner() {
		return neuerOrdner;
	}

	/**
	 * @return das LabelPart das den Ordnerr&uuml;cken darstellt
	 * @throws ApplicationException
	 */
	public LabelPart getOrdnerRuecken() throws ApplicationException {
		if (ordnerRuecken != null) {
			return ordnerRuecken; // NOPMD
		}

		ordnerRuecken = new LabelPart(labelSize, this);

		try {

			ordnerRuecken.setTitelText(ordner.getTitel());
			ordnerRuecken.setFont(LabelPartSectionsEnum.TITLE,
					ordner.getTitelFont());
			ordnerRuecken.setForegroundColor(LabelPartSectionsEnum.TITLE,
					ordner.getTitelColor());
			ordnerRuecken.setAlignment(LabelPartSectionsEnum.TITLE,
					ordner.getTitelAlignment());

			ordnerRuecken.setContentText(ordner.getContent());
			ordnerRuecken.setFont(LabelPartSectionsEnum.CONTENT,
					ordner.getContentFont());
			ordnerRuecken.setForegroundColor(LabelPartSectionsEnum.CONTENT,
					ordner.getContentColor());
			ordnerRuecken.setAlignment(LabelPartSectionsEnum.CONTENT,
					ordner.getContentAlignment());

			ordnerRuecken.setFooterText(ordner.getFooter());
			ordnerRuecken.setFont(LabelPartSectionsEnum.FOOTER,
					ordner.getFooterFont());
			ordnerRuecken.setForegroundColor(LabelPartSectionsEnum.FOOTER,
					ordner.getFooterColor());
			ordnerRuecken.setAlignment(LabelPartSectionsEnum.FOOTER,
					ordner.getFooterAlignment());

			ordnerRuecken.setIdText(ordner.getID());
			ordnerRuecken.setIdVisible(ordner.isIdVisible());

			ordnerRuecken.setBackgroundColor(ordner.getLabelBackground());
			ordnerRuecken.setBarColor(ordner.getBarColor());
			ordnerRuecken.setBarVisible(ordner.isBarVisible());

		} catch (RemoteException e) {
			throw new ApplicationException(
					I18N.tr("Error setting values for label"), e);
		}

		return ordnerRuecken;

	}

	/**
	 * @return Das Einstellpanel f&uuml;r das Register
	 * @throws ApplicationException
	 */
	public RegisterSettingsPart getRegisterSettingsPart()
			throws ApplicationException {

		if (registerSettings != null) {
			return registerSettings; // NOPMD
		}

		registerSettings = new RegisterSettingsPart(this);
		try {
			registerSettings.setIndexFont(ordner.getRegisterIndexFont());

			registerSettings.setIndexBgColor(ordner
					.getRegisterIndexBackground());
			registerSettings.setContentFont(ordner.getRegisterContentFont());
			registerSettings.setContentBgColor(ordner
					.getRegisterContentBackground());
		} catch (RemoteException e) {
			throw new ApplicationException(
					I18N.tr("Error setting values for register"), e);
		}
		return registerSettings;

	}

	/**
	 * @return die Registertabelle
	 * @throws RemoteException
	 */
	public RegisterTablePart getRegisterTabelle() throws RemoteException {
		if (registerTable != null) {
			return registerTable; // NOPMD
		}

		registerTable = new RegisterTablePart(register, null);
		registerTable.setContextMenu(new RegisterTableMenu(this));
		registerTable.addChangeListener(this);

		return registerTable;

	}

	/**
	 * Liefert den Print Button
	 * 
	 * @return Button f&uur das Drucken
	 */
	public Button getPrintButton() {
		if (printButton != null) {
			return printButton; // NOPMD
		}

		printButton = new Button(Settings.getI18n().tr("Print"), new Action() {

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {

				final List<Ordner> ordnerList = new ArrayList<>();
				ordnerList.add(ordner);
				try {
					final PrintDialog printDialog = new PrintDialog(ordnerList);
					printDialog.open();
				} catch (OperationCanceledException e) { // NOPMD
					// Hier machen wir nix :-)
				} catch (Exception e) { // NOPMD
					throw new ApplicationException(I18N
							.tr("Error opening PrintDialog"), e);
				}

			}
		});

		if (isNeuerOrdner()) {
			printButton.setEnabled(false);
		}

		return printButton;

	}

	/**
	 * @return den Speicher-Button
	 */
	public Button getSaveButton() {
		if (saveButton != null) {
			return saveButton; // NOPMD
		}

		saveButton = new Button(Settings.getI18n().tr("Save"), new Action() {

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {
				handleSave();
			}
		});
		saveButton.setEnabled(false);
		return saveButton;
	}

	/**
	 * @return den Abbruch-Button cancle
	 */
	public Button getCancelEditButton() {
		if (cancelEditButton != null) {
			return cancelEditButton; // NOPMD
		}

		cancelEditButton = new Button(Settings.getI18n().tr("Cancel"),
				new Action() {

					/**
					 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
					 */
					@Override
					public void handleAction(final Object context)
							throws ApplicationException {
						handleCancelEdit();
					}
				});
		return cancelEditButton;
	}

	/**
	 * Liefert eine Liste mit Buttons, um das Register neu zu bef&uuml;llen. Die
	 * List wird aus den RegisterTypeEnem Werten erstellt.
	 * 
	 * @return ein Liste mit Buttons, um das Register neu zu befllen.
	 */
	public List<ToolTipButton> getRegisterButtons() {
		if (registerSizeBtns != null) {
			return registerSizeBtns; // NOPMD
		}

		registerSizeBtns = new ArrayList<ToolTipButton>();

		for (final RegisterTypEnum registerType : RegisterTypEnum.values()) {
			@SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
			final ToolTipButton button = new ToolTipButton(
					registerType.getTitel(), new Action() {

						/**
						 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
						 */
						@Override
						public void handleAction(final Object context)
								throws ApplicationException {
							try {
								checkRegisterSize(registerType);
							} catch (RemoteException e) {
								throw new ApplicationException(
										I18N.tr("Error creating new register"),
										e);
							}
						}
					});
			button.setToolTip(registerType.getTooltip());
			registerSizeBtns.add(button);
		}
		return registerSizeBtns;
	}

	/**
	 * @return die Labelgre als String
	 */
	public String getLabelSize() {
		if (labelSize != null) {
			return labelSize.getValue(); // NOPMD
		}

		return "Nicht definiert";

	}

	/**
	 * Liefert ein labelInput mit einer Warnung zum Benutzen der rechten
	 * Maustaste innerhalb der Tabelle, falls das OS Linux ist, sonst ein leeres
	 * Feld
	 * 
	 * @return LabelInput mit Warnhinweis
	 */
	public String getLinuxWarning() {

		if (linuxWarning == null) {
			final String operationSystem = System.getProperty("os.name")
					.toLowerCase();

			if (operationSystem.indexOf("nix") >= 0
					|| operationSystem.indexOf("nux") >= 0
					|| operationSystem.indexOf("aix") > 0) {
				// Linux/Unix also warnung
				linuxWarning= I18N.tr("You are using Linux or Unix operating system") + ". "
						+ I18N.tr("There is a know bug for this OS") + ". "
						+ I18N.tr("Please avoid using the right-click on a table field which is in edit mode")  + ". "
						+ I18N.tr("If you did so and he screen seems frozen, please open the About dialog box from the main menu")  + ". ";
			} else {
				linuxWarning = "";
			}
		}

		return linuxWarning;

	}

	private void handleSave() throws ApplicationException {

		Logger.info(I18N.tr("Ordner speichern..."));

		// Erst mal den Ordner speichern
		try {

			ordner.setWidth(ordnerRuecken.getWidth());
			ordner.setTitel(getOrdnerRuecken().getTitelText());
			ordner.setTitelFont(getOrdnerRuecken().getFont(
					LabelPartSectionsEnum.TITLE));
			ordner.setTitelColor(getOrdnerRuecken().getForgroundColor(
					LabelPartSectionsEnum.TITLE));
			ordner.setTitelAlignment(getOrdnerRuecken().getAlignment(
					LabelPartSectionsEnum.TITLE));

			ordner.setContent(getOrdnerRuecken().getContentText());
			ordner.setContentFont(getOrdnerRuecken().getFont(
					LabelPartSectionsEnum.CONTENT));
			ordner.setContentColor(getOrdnerRuecken().getForgroundColor(
					LabelPartSectionsEnum.CONTENT));
			ordner.setContentAlignment(getOrdnerRuecken().getAlignment(
					LabelPartSectionsEnum.CONTENT));

			ordner.setFooter(getOrdnerRuecken().getFooterText());
			ordner.setFooterFont(getOrdnerRuecken().getFont(
					LabelPartSectionsEnum.FOOTER));
			ordner.setFooterColor(getOrdnerRuecken().getForgroundColor(
					LabelPartSectionsEnum.FOOTER));
			ordner.setFooterAlignment(getOrdnerRuecken().getAlignment(
					LabelPartSectionsEnum.FOOTER));

			ordner.setIdVisible(getOrdnerRuecken().isIdVisible());

			ordner.setLabelBackground(getOrdnerRuecken().getBackgroundColor());
			ordner.setBarColor(getOrdnerRuecken().getBarColor());
			ordner.setBarVisible(getOrdnerRuecken().getBarVisible());

			ordner.setRegisterIndexFont(getRegisterSettingsPart()
					.getIndexFont());
			ordner.setRegisterIndexBackground(getRegisterSettingsPart()
					.getIndexBgColor());
			ordner.setRegisterContentFont(getRegisterSettingsPart()
					.getContentFont());
			ordner.setRegisterContentBackground(getRegisterSettingsPart()
					.getContentBgColor());

			ordner.store();

			// Speichern vom Register
			final ArrayList<String> registerIds = new ArrayList<>();
			final List<?> regList = registerTable.getItems();

			for (final Object object : regList) {
				final Register registerEintrag = (Register) object;
				registerEintrag.setOrdner(ordner);
				registerEintrag.store();
				registerIds.add(registerEintrag.getID());
			}

			// Entfernen von gelschten Registern
			final DBIterator listRegister = ordner.getInhaltsverzeichniss();
			while (listRegister.hasNext()) {
				final Register registerEintrag = (Register) listRegister.next();
				if (!registerIds.contains(registerEintrag.getID())) {
					registerEintrag.delete();
				}
			}

			neuerOrdner = false;
			inhaltChanged = false;
			updateActionButtons();

		} catch (RemoteException e) {
			throw new ApplicationException(
					I18N.tr("Folder could not be saved"), e);
		}

	}

	private void handleCancelEdit() {

		inhaltChanged = false;
		updateActionButtons();
		GUI.startView(OrdnerListView.class.getName(), null);

	}

	private void updateActionButtons() {
		if (inhaltChanged) {
			getSaveButton().setEnabled(true);
			getPrintButton().setEnabled(false);
		} else {
			getSaveButton().setEnabled(false);
			getPrintButton().setEnabled(true);
		}
	}

	private void checkRegisterSize(final RegisterTypEnum typ)
			throws RemoteException {

		this.checkRegisterSize(typ.getIndex().length, typ);
	}

	private void checkRegisterSize(final int rows, final RegisterTypEnum typ)
			throws RemoteException {

		String[] index;
		if (typ == null) {
			index = new String[rows];
			Arrays.fill(index, "");
		} else {
			index = typ.getIndex();
		}

		@SuppressWarnings("unchecked")
		final List<Register> items = (List<Register>) registerTable.getItems();
		registerTable.removeAll();

		final DBService service = Settings.getDBService();

		if (rows < items.size()) {
			for (int i = items.size() - 1; i >= rows; i--) {
				items.remove(i);
			}
		}

		for (int i = 0; i < rows; i++) {

			Register reg;
			if (i < items.size()) {
				reg = (Register) items.get(i);
				reg.setIndex(index[i]);
			} else {
				reg = (RegisterImpl) service.createObject(Register.class, null);

				reg.setIndex(index[i]);
				reg.setBeschreibung("leer");
				reg.setOrdner(ordner);
				items.add(reg);
			}

			if (Logger.isLogging(Level.DEBUG)) {
				Logger.debug(("RegBe: " + reg.getBeschreibung()));
			}

		}

		for (final Register reg1 : items) {
			registerTable.addItem(reg1);
		}

		if (!inhaltChanged) {
			inhaltChanged = true;
			updateActionButtons();
		}

	}

	/**
	 * @see de.willuhn.jameica.gui.parts.TableChangeListener#itemChanged(java.lang
	 *      .Object, java.lang.String, java.lang.String)
	 */
	@Override
	public void itemChanged(final Object object, final String attribute,
			final String newValue) throws ApplicationException {

		if (object instanceof Register) {
			final Register eintrag = (Register) object;

			try {

				if (Logger.isLogging(Level.DEBUG)) {
					Logger.debug((eintrag.getBeschreibung() + " - " + newValue));
				}

				switch (attribute) {
				case "idx":
					eintrag.setIndex(newValue);
					break;
				case "content":
					eintrag.setBeschreibung(newValue);
					break;
				default:
					break;
				}

				if (!inhaltChanged) {
					inhaltChanged = true;
					updateActionButtons();
				}

			} catch (RemoteException e) {
				throw new ApplicationException(
						I18N.tr("Update of register entry failed"), e);
			}

		}

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsListener#notifyButtonClicked
	 *      (de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsButtonObject)
	 */
	@Override
	public void notifyButtonClicked(
			final RegisterSettingsButtonObject buttonObject) {

		boolean setChange = true;
		RegisterSettingsButtonsEnum button;
		button = buttonObject.getButton();

		FontDialog fontDialog;
		FontData fontData;
		ColorDialog colorDialog;
		RGB colorRgb;
		Color color;
		Font font;

		switch (button) {
		case INDEX_FONT:
			fontDialog = new FontDialog(GUI.getShell());
			fontDialog.setEffectsVisible(false);
			fontDialog.setFontList(buttonObject.getFont().getFontData());
			fontData = fontDialog.open();
			if (fontData != null) {
				font = new Font(GUI.getDisplay(), fontData);
				registerSettings.setIndexFont(font);
			}
			break;
		case INDEX_BACKGROUND_COLOR:
			colorDialog = new ColorDialog(GUI.getShell());
			colorDialog.setRGB(buttonObject.getColor().getRGB());
			colorRgb = colorDialog.open();
			if (colorRgb != null) {
				color = new Color(GUI.getDisplay(), colorRgb);
				registerSettings.setIndexBgColor(color);
			}
			break;
		case CONTENT_FONT:
			fontDialog = new FontDialog(GUI.getShell());
			fontDialog.setEffectsVisible(false);
			fontDialog.setFontList(buttonObject.getFont().getFontData());
			fontData = fontDialog.open();
			if (fontData != null) {
				font = new Font(GUI.getDisplay(), fontData);
				registerSettings.setContentFont(font);
			}
			break;
		case CONTENT_BACKGROUND_COLOR:
			colorDialog = new ColorDialog(GUI.getShell());
			colorDialog.setRGB(buttonObject.getColor().getRGB());
			colorRgb = colorDialog.open();
			if (colorRgb != null) {
				color = new Color(GUI.getDisplay(), colorRgb);
				registerSettings.setContentBgColor(color);
			}
			break;
		default:
			setChange = false;
			break;

		}

		if (setChange && !inhaltChanged) {
			inhaltChanged = true;
			updateActionButtons();
		}
	}

	/**
	 * @see de.frankmuenster.jameica.OrVerw.Parts.LabelPart.LabelPartListener#
	 *      notifyButtonClicked
	 *      (de.frankmuenster.jameica.OrVerw.Parts.LabelPart.LabelPartButtonObject)
	 */
	@Override
	public void notifyButtonClicked(final LabelPartButtonObject buttonObject) {

		Boolean checkChange = true;
		final LabelPartButtonsEnum button = buttonObject.getButton();

		ColorDialog colorDialog;
		RGB colorRgb;
		Color color;

		switch (button) {
		case ALIGN_CENTER:
			ordnerRuecken.setAlignment(buttonObject.getSection(), SWT.CENTER);
			break;
		case ALIGN_LEFT:
			ordnerRuecken.setAlignment(buttonObject.getSection(), SWT.LEFT);
			break;
		case ALIGN_RIGHT:
			ordnerRuecken.setAlignment(buttonObject.getSection(), SWT.RIGHT);
			break;
		case FONT:
			final FontDialog fontDialog = new FontDialog(GUI.getShell());
			fontDialog.setEffectsVisible(false);
			fontDialog.setFontList(buttonObject.getFont().getFontData());
			final FontData fontData = fontDialog.open();
			if (fontData != null) {
				final Font font = new Font(GUI.getDisplay(), fontData);
				ordnerRuecken.setFont(buttonObject.getSection(), font);
			}
			break;
		case COLOR:
			colorDialog = new ColorDialog(GUI.getShell());
			colorDialog.setRGB(buttonObject.getColor().getRGB());
			colorRgb = colorDialog.open();
			if (colorRgb != null) {
				color = new Color(GUI.getDisplay(), colorRgb);
				ordnerRuecken.setForegroundColor(buttonObject.getSection(),
						color);
			}
			break;
		case BACKGROUND_COLOR:
			colorDialog = new ColorDialog(GUI.getShell());
			colorDialog.setRGB(buttonObject.getColor().getRGB());
			colorRgb = colorDialog.open();
			if (colorRgb != null) {
				color = new Color(GUI.getDisplay(), colorRgb);
				ordnerRuecken.setBackgroundColor(color);
			}
			break;
		case BAR_COLOR:
			colorDialog = new ColorDialog(GUI.getShell());
			colorDialog.setRGB(buttonObject.getColor().getRGB());
			colorRgb = colorDialog.open();
			if (colorRgb != null) {
				color = new Color(GUI.getDisplay(), colorRgb);
				ordnerRuecken.setBarColor(color);
			}
			break;
		case BAR_VISIBLE:
			ordnerRuecken.setBarVisible(!ordnerRuecken.getBarVisible());
			break;
		case ID_VISIBLE:
			ordnerRuecken.setIdVisible(!ordnerRuecken.isIdVisible());
			break;
		default:
			checkChange = false;
			break;

		}

		if (checkChange && !inhaltChanged) {
			inhaltChanged = true;
			updateActionButtons();
		}

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartListener#notifyLabelTextChanged()
	 */
	@Override
	public void notifyLabelTextChanged() {
		if (!inhaltChanged) {
			inhaltChanged = true;
			updateActionButtons();
		}
	}

}
