//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw.gui.controller;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.gui.action.OrdnerNewAction;
import de.frankmuenster.jameica.orverw.gui.action.OrdnerOpenAction;
import de.frankmuenster.jameica.orverw.gui.action.ViewSettingsAction;
import de.frankmuenster.jameica.orverw.gui.menu.OrdnerTableMenu;
import de.frankmuenster.jameica.orverw.parts.OrdnerTablePart;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.util.I18N;

/**
 * Kontroller f&uuml; die Ordner&uuml;bersicht
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
@SuppressWarnings("PMD.BeanMembersShouldSerialize")
public class OrdnerListViewController extends AbstractControl {

	/**
	 * Unser &Uuml;bersetzer
	 */
	private final static I18N I18N = Settings.getI18n();

	/**
	 * Die Ordner-Tabelle
	 */
	private TablePart ordnerTabelle;

	/**
	 * Button zum erstellen eines neuen Ordners
	 */
	private Button neuerOrdnerButton;

	/**
	 * Einstellungs Button
	 */
	private Button settinsgButton;

	/**
	 * Kontroller-Konstruktor
	 * 
	 * @param view
	 */
	public OrdnerListViewController(final AbstractView view) {
		super(view);
	}

	/**
	 * Liefert das Ordner-Tabelle-Part Objekt
	 * 
	 * @return die Ordner-Tabelle
	 * @throws RemoteException
	 */
	public TablePart getOrdnerTabelle() throws RemoteException {
		if (ordnerTabelle != null) {
			return ordnerTabelle; // NOPMD
		}

		ordnerTabelle = new OrdnerTablePart(new OrdnerOpenAction());
		ordnerTabelle.setContextMenu(new OrdnerTableMenu());
		return ordnerTabelle;
	}

	/**
	 * Liefert einen 'Neuen Ordner' Button.
	 * 
	 * @return Button zum erstellen eines neuen Ordners
	 */
	public Button getNeuerOrdner() {
		if (neuerOrdnerButton != null) {
			return neuerOrdnerButton; // NOPMD
		}

		final Action action = new OrdnerNewAction();
		neuerOrdnerButton = new Button(I18N.tr("Create New Folder"), action, null,
				false, "scanner-icon.png");
		neuerOrdnerButton.setEnabled(true);
		return neuerOrdnerButton;

	}

	/**
	 * Liefert einen 'Einstellung' Button.
	 * 
	 * @return Button zum erstellen eines neuen Ordners
	 */
	public Button getSettings() {
		if (settinsgButton == null) {
			final Action action = new ViewSettingsAction();
			settinsgButton = new Button(I18N.tr("Settings"), action, null,
					false, "scanner-icon.png");
			settinsgButton.setEnabled(true);
		}
		return settinsgButton;
	}
}
