package de.frankmuenster.jameica.orverw.gui.controller;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.gui.input.TextStyle;
import de.frankmuenster.jameica.orverw.gui.input.TextStyleInput;
import de.frankmuenster.jameica.orverw.gui.views.SettingsView;
import de.frankmuenster.jameica.orverw.parts.HorizontalLinePart;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Kontroller fr die Settings View
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class SettingsController extends AbstractControl {

	/**
	 *  Unser &Uuml;bersetzer
	 */
	private final static I18N I18N = Settings.getI18n();

	/**
	 * Das Part f&uuml;r die obere Linie
	 */
	private transient HorizontalLinePart barUpper;
	/**
	 * Das Part f&uuml;r die untere Linie
	 */
	private transient HorizontalLinePart barLower;
	/**
	 * Die Farbe der Linien
	 */
	private transient Color barColor;
	/**
	 * Der Button f&uuml; die Linienfarbe
	 */
	private transient Button barColorButton;

	/**
	 * Die Sichbarkeitscheckbox der Linien
	 */
	private transient CheckboxInput barVisible;

	/**
	 * Der Texttyp des Inhaltbereiches
	 */
	private transient TextStyleInput contentStyle;

	/**
	 * Der TZexttype des Fu&szlig;bereiches
	 */
	private transient TextStyleInput footerStyle;

	/**
	 * Beipsieltext des ID Feldes
	 */
	private transient TextInput idExample;
	/**
	 * Checkbox ID Sichtbar
	 */
	private transient CheckboxInput idVisible;

	/**
	 * Hintergrundfarbe des Labels
	 */
	private transient Color lblBgColor;
	/**
	 * Buttons f&uuml; die Hintergrundfarbe des Labels
	 */
	private transient Button lblBgButton;

	/**
	 * Texttyp desd Inhaltbereiches
	 */
	private transient TextStyleInput regContentStyle;

	/**
	 * Texttyp des Registerindex
	 */
	private transient TextStyleInput regIndexStyle;

	/**
	 * Texttyp des Titelbereiches
	 */
	private transient TextStyleInput titleSytle;

	/**
	 * Erstellt eine neue Instanz des Kontrollers
	 * 
	 * @param view
	 */
	public SettingsController(final SettingsView view) {
		super(view);
		this.view = view;

	}

	/**
	 * Button f&uuml;r die Bar-Hintergrundfarbe
	 * 
	 * @return
	 */
	public Button getBarColorButton() {
		if (barColorButton != null) {
			return barColorButton; //NOPMD
		}

		barColor = Settings.getDefaultBarColor();
		barColorButton = new Button(I18N.tr("Line Color"), new Action() {

			@Override
			public void handleAction(final Object context) // NOPMD
					throws ApplicationException {

				final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
				colorDialog.setRGB(barColor.getRGB());
				colorDialog.setText(Application.getI18n().tr(
						"Bitte whlen Sie die Farbe aus"));
				barColor = new Color(GUI.getDisplay(), colorDialog.open());

				// Update bar colors
				if (barUpper != null) {
					barUpper.setColor(barColor);
				}
				if (barLower != null) {
					barLower.setColor(barColor);
				}

			}
		});

		// Update demo lines colors
		if (barUpper != null) {
			barUpper.setColor(barColor);
		}
		if (barLower != null) {
			barLower.setColor(barColor);
		}

		return barColorButton;
	}

	/**
	 * @return die die HorizontalLinePart fr die untere DemoLinie.
	 */
	public HorizontalLinePart getBarLower() {
		if (barLower != null) {
			return barLower; //NOPMD
		}

		final Point size = new Point(200, 8);
		barLower = new HorizontalLinePart((Color) barColor, size, false);

		if (barVisible != null) {
			barLower.setVisible((boolean) barVisible.getValue());
		}
		return barLower;
	}

	/**
	 * @return die HorizontalLinePart fr die obere DemoLinie.
	 */
	public HorizontalLinePart getBarUpper() {
		if (barUpper != null) {
			return barUpper; // NOPMD
		}

		final Point size = new Point(200, 8);
		barUpper = new HorizontalLinePart((Color) barColor, size, false);

		if (barVisible != null) {
			barUpper.setVisible((boolean) barVisible.getValue());
		}

		return barUpper;
	}

	/**
	 * @return Ein CheckBoxInput fr Trennlinien-Sichtbarkeit.
	 */
	public CheckboxInput getBarVisible() {
		if (barVisible != null) {
			return barVisible; // NOPMD
		}

		barVisible = new CheckboxInput(Settings.isDefaultBarVisible());
		barVisible.setName(I18N.tr("Lines Visible"));

		barVisible.addListener(new Listener() {

			@Override
			public void handleEvent(final Event arg0) { // NOPMD
				// Set visibility of demo lines
				if (barUpper != null) {
					barUpper.setVisible((boolean) barVisible.getValue());
				}
				if (barLower != null) {
					barLower.setVisible((boolean) barVisible.getValue());
				}
			}
		});

		// Set visibility of demo lines
		if (barUpper != null) {
			barUpper.setVisible((boolean) barVisible.getValue());
		}
		if (barLower != null) {
			barLower.setVisible((boolean) barVisible.getValue());
		}

		return barVisible;
	}

	/**
	 * @return TextStyleInput f&uuml; den Inhaltsbereich.
	 * @throws ApplicationException
	 */
	public TextStyleInput getContentStyle() throws ApplicationException {
		if (contentStyle != null) {
			return contentStyle; // NOPMD
		}

		final TextStyle content = Settings.getDefaultContentStyle();
		content.setBackgroundColor(lblBgColor);
		contentStyle = new TextStyleInput(content,
				TextStyleInput.ALIGNMENT | TextStyleInput.FOREGROUND);
		return contentStyle;
	}

	/**
	 * @return TextStyleInput f&uuml;r den Fussbereich.
	 * @throws ApplicationException
	 */
	public TextStyleInput getFooterStyle() throws ApplicationException {
		if (footerStyle != null) {
			return footerStyle; // NOPMD
		}
		
		final TextStyle footer = Settings.getDefaultFooterStyle();
		footer.setBackgroundColor(lblBgColor);
		footerStyle = new TextStyleInput(footer,
				TextStyleInput.ALIGNMENT | TextStyleInput.FOREGROUND);
		return footerStyle;
	}

	/**
	 * @return Input f&uuml;r das ID-Beispiel
	 */
	public TextInput getIdExample() {
		if (idExample != null) {
			return idExample; // NOPMD
		}

		idExample = new TextInput("99");
		idExample.setEnabled(false);

		return idExample;

	}

	/**
	 * @return Input fr die ID-Sichtbarkeit
	 */
	public CheckboxInput getIdVisible() {
		if (idVisible != null) {
			return idVisible; // NOPMD
		}

		idVisible = new CheckboxInput(Settings.isDefaultIdVisible());
		idVisible.setName("ID Sichtbar");
		idVisible.addListener(new Listener() {

			@Override
			public void handleEvent(final Event arg0) { // NOPMD
				if (idExample != null) {
					idExample.getControl().setVisible(
							(Boolean) idVisible.getValue());
				}
			}
		});

		return idVisible;
	}

	/**
	 * Button f&uuml;r die Label-Hintergrundfarbe
	 * 
	 * @return
	 */
	public Button getLabelBackgroundButton() {
		if (lblBgButton != null) {
			return lblBgButton; // NOPMD
		}

		lblBgColor = Settings.getDefaultLabelBackground();
		lblBgButton = new Button(I18N.tr("Label Background Color"),
				new Action() {

					@Override
					public void handleAction(final Object context) // NOPMD
							throws ApplicationException {

						final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
						colorDialog.setRGB(lblBgColor.getRGB());
						colorDialog.setText(I18N.tr("Bitte whlen Sie die Farbe aus"));
						lblBgColor = new Color(GUI.getDisplay(), colorDialog
								.open());

						// Update example text background colors
						if (titleSytle != null) {
							final TextStyle style = (TextStyle) titleSytle.getValue();
							style.setBackgroundColor(lblBgColor);
							titleSytle.setValue(style);
						}
						if (contentStyle != null) {
							final TextStyle style = (TextStyle) contentStyle
									.getValue();
							style.setBackgroundColor(lblBgColor);
							contentStyle.setValue(style);
						}
						if (footerStyle != null) {
							final TextStyle style = (TextStyle) footerStyle
									.getValue();
							style.setBackgroundColor(lblBgColor);
							footerStyle.setValue(style);
						}
					}
				});

		return lblBgButton;
	}

	/**
	 * @return TextStyleInput f&uuml;r den Register-Inhalt
	 * @throws ApplicationException
	 */
	public TextStyleInput getRegisterContentStyle() throws ApplicationException {
		if (regContentStyle != null) {
			return regContentStyle; // NOPMD
		}

		regContentStyle = new TextStyleInput(
				Settings.getDefaultRegisterContentStyle(),
				TextStyleInput.BACKGROUND | TextStyleInput.FOREGROUND,
				new Point(100, 0));
		return regContentStyle;
	}

	/**
	 * @return TextStyleInput f&uuml;r den Register-Index
	 * @throws ApplicationException
	 */
	public TextStyleInput getRegisterIndexStyle() throws ApplicationException {
		if (regIndexStyle != null) {
			return regIndexStyle; // NOPMD
		}

		regIndexStyle = new TextStyleInput(
				Settings.getDefaultRegisterIndexStyle(),
				TextStyleInput.BACKGROUND | TextStyleInput.FOREGROUND,
				new Point(100, 0));
		return regIndexStyle;
	}

	/**
	 * @return Ein TextStyleInput fr den Titel-Style.
	 * @throws ApplicationException
	 */
	public TextStyleInput getTitleStyle() throws ApplicationException {
		if (titleSytle != null) {
			return titleSytle; // NOPMD
		}

		final TextStyle title = Settings.getDefaultTitleStyle();
		title.setBackgroundColor(lblBgColor);
		titleSytle = new TextStyleInput(title,
				TextStyleInput.ALIGNMENT | TextStyleInput.FOREGROUND);
		return titleSytle;
	}

	/**
	 * Speichert die aktuellen Einstellungen im Settings Objekt.
	 */
	public void speichern() {

		Settings.setDefaultBarColor((Color) barColor);
		Settings.setDefaultBarVisible((Boolean) barVisible.getValue());
		Settings.setDefaultContentStyle((TextStyle) contentStyle.getValue());
		Settings.setDefaultFooterStyle((TextStyle) footerStyle.getValue());
		Settings.setDefaultIdVisible((Boolean) idVisible.getValue());
		Settings.setDefaultLabelBackground(lblBgColor);
		Settings.setDefaultRegisterContentStyle((TextStyle) regContentStyle
				.getValue());
		Settings.setDefaultRegisterIndexStyle((TextStyle) regIndexStyle
				.getValue());
		Settings.setDefaultTitleStyle((TextStyle) titleSytle.getValue());

		GUI.getStatusBar().setSuccessText("Einstellungen gespeichert.");

	}

}
