/*
* Copyright 2016 frankmuenster.de
*
* Lizenziert unter der EUPL, Version 1.1 oder - sobald
* diese von der Europischen Kommission genehmigt wurden -
* Folgeversionen der EUPL ("Lizenz");
* Sie drfen dieses Werk ausschlielich gem
* dieser Lizenz nutzen.
* Eine Kopie der Lizenz finden Sie hier:
*
*   http://ec.europa.eu/idabc/eupl
*
* Sofern nicht durch anwendbare Rechtsvorschriften
* gefordert oder in schriftlicher Form vereinbart, wird
* die unter der Lizenz verbreitete Software "so wie sie
* ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
* ausdrcklich oder stillschweigend - verbreitet.
* Die sprachspezifischen Genehmigungen und Beschrnkungen
* unter der Lizenz sind dem Lizenztext zu entnehmen.
*/ 
package de.frankmuenster.jameica.orverw.gui.dialog;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelSizeEnum;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 *
 */
public class NeuerOrdnerDialog extends AbstractDialog<LabelSizeEnum> {

	/**
	 * Die Bezeichnung der Radio BUttun Gruppe
	 */
	private static final String RADIO_GROUP_ID = "labelSize"; 
	
	/**
	 * Der R&uuml;ckgabewert des Dialogfensters
	 */
	private transient LabelSizeEnum result;

	
	/**
	 * Dialog zum Auswahl der Breite eines neuen Ordners
	 */
	public NeuerOrdnerDialog() {
		super(AbstractDialog.POSITION_CENTER, false);
		this.setTitle(Settings.getI18n().tr("Neuer Ordner"));
		//this.setSize(270, 160);
	}
	
	/**
	 * @param position
	 */
	private NeuerOrdnerDialog(final int position) {
		super(position);
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) throws Exception { // NOPMD

		final GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		parent.setLayout(layout);

		// Erstelle einfachen Container
		final Container container = new SimpleContainer(parent, true, 1);
		
		RadioInput sizeRadioInput;
		for (final LabelSizeEnum labelsize : LabelSizeEnum.values()) {
			sizeRadioInput = new RadioInput(RADIO_GROUP_ID, true); // NOPMD
			sizeRadioInput.setName(labelsize.getValue());
			sizeRadioInput.setValue(labelsize);
			sizeRadioInput.paint(container.getComposite());
		}
		//String defaultLabel = LabelSizeEnum.getDefault().getValue();
		RadioInput.select(RADIO_GROUP_ID, LabelSizeEnum.getDefault());
		
		
		final ButtonArea buttons = new ButtonArea();
		buttons.addButton(Settings.getI18n().tr("Erstellen"), new Action() {
			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {
				result = (LabelSizeEnum) RadioInput.getValue(RADIO_GROUP_ID);
				close();
			}
		}, null, true, "ok.png");
		
		buttons.addButton(Settings.getI18n().tr("Cancel"), new Action() {
			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {
				result = null; // NOPMD
				close();
			}
		}, null, false, "window-close.png");

		container.addButtonArea(buttons);
		
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected LabelSizeEnum getData() { 
		return result;
	}

}
