/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.gui.dialog;

import java.util.ArrayList;
import java.util.List;

import net.sf.paperclips.Margins;
import net.sf.paperclips.PaperClips;
import net.sf.paperclips.PrintJob;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.print.AbstractPrint;
import de.frankmuenster.jameica.orverw.print.PrintRegister;
import de.frankmuenster.jameica.orverw.print.PrintLabel;
import de.frankmuenster.jameica.orverw.print.PrintViewDialog;
import de.frankmuenster.jameica.orverw.util.TextUtils;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
public class PrintDialog extends AbstractDialog<Boolean> {

	/**
	 * Liste aller Ordner die angezeigt werden
	 */
	final private transient List<Ordner> ordnerList;
	/**
	 * Liste der Ordner mit den Print-Informationen
	 */
	private transient List<PrintInfo> printInfoList;

	/**
	 * R&uuml;ckgabewert des Dialoges
	 */
	// private Boolean result;

	/**
	 * Bean Klasse mit den Printinformationen
	 * 
	 * @author munstfr
	 * 
	 */
	private class PrintInfo {

		/**
		 * Der Ordner
		 */
		public transient Ordner ordner;
		/**
		 * Soll der Ordnerr&uuml;cken-Preview angezeigt werden
		 */
		public transient Button showLabelPreView;
		/**
		 * Soll der Ordnerr&uuml;cken gedruckt werden
		 */
		public transient Button printLabel;
		/**
		 * Soll das Register angezeigt werden
		 */
		public transient Button showRegPreView;
		/**
		 * Soll das Register gedruckt werden
		 */
		public transient Button printRegister;

		/**
		 * Standart Konstruktor
		 */
		public PrintInfo() { // NOPMD
			// empty constructor
		}

	}

	/**
	 * @param position
	 */
	public PrintDialog(final List<Ordner> ordnerList) {
		super(AbstractDialog.POSITION_CENTER, false);
		this.setTitle(Settings.getI18n().tr("Print"));
		this.setSize(450, 300);

		this.ordnerList = ordnerList;

	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets
	 *      .Composite)
	 */
	@Override
	@SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
	protected void paint(Composite parent) throws Exception { // NOPMD

		final GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		parent.setLayout(layout);

		// Erstelle einfachen Container
		final Container containerMain = new SimpleContainer(parent, true, 1);

		// Erstelle einfachen Container
		final Container container = new SimpleContainer(
				containerMain.getComposite(), true, 6);

		printInfoList = new ArrayList<>();

		Button checkbox;
		Label label;

		label = new Label(container.getComposite(), SWT.NONE);
		label.setAlignment(SWT.CENTER);
		label.setText("ID");
		label = new Label(container.getComposite(), SWT.NONE);
		label.setAlignment(SWT.CENTER);
		label.setText("Titel");
		label = new Label(container.getComposite(), SWT.NONE);
		GridData griddata = new GridData();
		griddata.horizontalSpan = 2;
		griddata.horizontalAlignment = SWT.CENTER;
		label.setLayoutData(griddata);
		label.setAlignment(SWT.CENTER);
		label.setText("Label");
		label = new Label(container.getComposite(), SWT.NONE);
		griddata = new GridData();
		griddata.horizontalSpan = 2;
		griddata.horizontalAlignment = SWT.CENTER;
		label.setLayoutData(griddata);
		label.setText("Register");

		label = new Label(container.getComposite(), SWT.NONE);
		griddata = new GridData();
		griddata.horizontalSpan = 2;
		label.setLayoutData(griddata);
		label.setText("");
		label = new Label(container.getComposite(), SWT.NONE);
		label.setAlignment(SWT.CENTER);
		label.setText("Vorschau");
		label = new Label(container.getComposite(), SWT.NONE);
		label.setAlignment(SWT.CENTER);
		label.setText("Drucken");
		label = new Label(container.getComposite(), SWT.NONE);
		label.setAlignment(SWT.CENTER);
		label.setText("Vorschau");
		label = new Label(container.getComposite(), SWT.NONE);
		label.setAlignment(SWT.CENTER);
		label.setText("Drucken");

		for (final Ordner ordner : ordnerList) {

			final PrintInfo printInfo = new PrintInfo();
			printInfo.ordner = ordner;

			label = new Label(container.getComposite(), SWT.NONE);
			label.setText(ordner.getID());

			label = new Label(container.getComposite(), SWT.NONE);
			griddata = new GridData(); // NOPM
			griddata.grabExcessHorizontalSpace = true;
			label.setLayoutData(griddata);
			label.setText(TextUtils.format(ordner.getTitel(), 20));

			checkbox = new Button(container.getComposite(), SWT.CHECK);
			griddata = new GridData(); //
			griddata.horizontalAlignment = SWT.CENTER;
			checkbox.setLayoutData(griddata);
			checkbox.setSelection(false);
			printInfo.showLabelPreView = checkbox;

			checkbox = new Button(container.getComposite(), SWT.CHECK);
			griddata = new GridData();
			griddata.horizontalAlignment = SWT.CENTER;
			checkbox.setLayoutData(griddata);
			checkbox.setSelection(true);
			printInfo.printLabel = checkbox;

			checkbox = new Button(container.getComposite(), SWT.CHECK);
			griddata = new GridData();
			griddata.horizontalAlignment = SWT.CENTER;
			checkbox.setLayoutData(griddata);
			checkbox.setSelection(false);
			printInfo.showRegPreView = checkbox;

			checkbox = new Button(container.getComposite(), SWT.CHECK);
			griddata = new GridData();
			griddata.horizontalAlignment = SWT.CENTER;
			checkbox.setLayoutData(griddata);
			checkbox.setSelection(true);
			printInfo.printRegister = checkbox;

			printInfoList.add(printInfo);
		}

		final ButtonArea buttons = new ButtonArea();
		buttons.addButton(Settings.getI18n().tr("Print"), new Action() {
			@Override
			public void handleAction(final Object context) // NOPMD
					throws ApplicationException {
				handlePrint();
				close();
			}
		}, null, true, "ok.png");

		buttons.addButton(Settings.getI18n().tr("Cancel"), new Action() {
			@Override
			public void handleAction(final Object context) // NOPMD
					throws ApplicationException {
				// result = false;
				close();
			}
		}, null, false, "window-close.png");

		containerMain.addButtonArea(buttons);

	}

	@SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
	private void handlePrint() throws ApplicationException { // NOPMD

		for (final PrintInfo printInfo : printInfoList) {
			// Label
			if (printInfo.showLabelPreView.getSelection()) {
				Logger.debug("Show label preview");
				final PrintLabel printLabel = new PrintLabel(printInfo.ordner);
				showPreView(printLabel);
			} else if (printInfo.printLabel.getSelection()) {
				Logger.debug("print label");
				final PrintLabel printLabel = new PrintLabel(printInfo.ordner);
				startPrint(printLabel);
			}

			// Register
			if (printInfo.showRegPreView.getSelection()) {
				Logger.debug("Show register preview");
				final PrintRegister printIndex = new PrintRegister(printInfo.ordner);
				showPreView(printIndex);
			} else if (printInfo.printRegister.getSelection()) {
				Logger.debug("print register");
				final PrintRegister printIndex = new PrintRegister(printInfo.ordner);
				startPrint(printIndex);
			}
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected Boolean getData() throws Exception { // NOPMD
		return null;
	}

	private void showPreView(final AbstractPrint print)
			throws ApplicationException {

		final PrintViewDialog dialog = new PrintViewDialog(print,
				this.getShell());

		try {
			dialog.open();
		} catch (Exception e) { // NOPMD
			throw new ApplicationException("Error showing print preview", e);
		}
	}

	private void startPrint(final AbstractPrint print)
			throws ApplicationException {
		final org.eclipse.swt.printing.PrintDialog dialog = new org.eclipse.swt.printing.PrintDialog(
				this.getShell(), SWT.NONE);
		final PrinterData printerData = dialog.open();
		if (printerData != null) {

			// Erstelle Printjob
			final PrintJob job = new PrintJob("OrdnerVerwaltung-"
					+ print.getTitle(), print.createPrint());

			final Margins margins = job.getMargins();
			margins.left = 100;
			margins.right = 50;

			// Drucken starten
			PaperClips.print(job, printerData);
		}
	}

}
