/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.gui.menu;

import java.rmi.RemoteException;
import java.util.List;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.db.server.RegisterImpl;
import de.frankmuenster.jameica.orverw.gui.controller.OrdnerBearbeitenController;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.parts.CheckedContextMenuItem;
import de.willuhn.jameica.gui.parts.ContextMenu;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.util.ApplicationException;

/**
 * Kontextmen&uuml; der Registertabelle
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class RegisterTableMenu extends ContextMenu {
	
	/**
	 * Kontroller zum Bearbeiten von Ordner
	 */
	final private transient OrdnerBearbeitenController obc;

	/**
	 * Konstruktor f&uuml;r das Kontextmen&uuml; der Registertabelle
	 * 
	 * @param obc
	 *            der Kontroller zum Bearbeiten von Ordner
	 */
	public RegisterTableMenu(final OrdnerBearbeitenController obc) {
		
		super();
		this.obc = obc;
		
		// CheckedContextMenuItems will be disabled, if the user clicks into
		// an empty space of the table
		addItem(new CheckedContextMenuItem(Settings.getI18n().tr(
				"Add line below..."), new Action() {

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {
				if (context instanceof Register) {
					handleAddLine((Register) context);
				}
			}
		}, "list-add.png"));

		addItem(new CheckedContextMenuItem(Settings.getI18n().tr(
				"Remove line..."), new Action() {

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {

				if (context instanceof Register) {
					handleRemoveLine(context);
				}
			}
		}, "list-remove.png"));

	}

	private void handleAddLine (final Register register) throws ApplicationException  {
			try {
				final TablePart table = obc.getRegisterTabelle();

				for (int i = 0; i < table.getItems().size(); i++) {
					if (register.equals(table.getItems().get(i))) {

						final DBService service = Settings.getDBService();
						final Register reg = (RegisterImpl) service
								.createObject(Register.class, null);
						reg.setOrdner(register.getOrdner());
						table.addItem(reg, i + 1);
						break;
					}
				}
			} catch (RemoteException e) {
				throw new ApplicationException(Settings.getI18n().tr(
						"Error adding line."), e);
			}

		
	}
	
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	private void handleRemoveLine (final Object context) throws ApplicationException {
		try {
			final TablePart table = obc.getRegisterTabelle();

			if (table.getItems() instanceof List<?>) {
				final List<?> registerEntries = table.getItems();
				registerEntries.remove(context);

				table.removeAll();
				for (final Object entry : registerEntries) {
					table.addItem(entry);
				}
			}
		} catch (RemoteException e) {
			throw new ApplicationException(Settings.getI18n().tr(
					"Error removing line."), e);
		}

	}
	
}
