/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.orverw.gui.views;

import org.eclipse.swt.layout.GridLayout;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.gui.action.ViewOrdnerVerwaltungAction;
import de.frankmuenster.jameica.orverw.gui.controller.SettingsController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * View fr die Einstellungen
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class SettingsView extends AbstractView {

	/**
	 * Unser &Uuml;bersetzer
	 */
	private final static I18N I18N = Settings.getI18n();

	/**
	 * Der Tab-Folder, in dem die einzelnen Module der Einstellungen
	 * untergebracht sind.
	 */
	//private transient TabFolder folder;

	/**
	 * Unser Standartkonstruktor
	 */
	@SuppressWarnings("PMD.UnnecessaryConstructor")
	public SettingsView() {
		super();
		// Intentionally left blank
	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#bind()
	 */
	@Override
	@SuppressWarnings("PMD.SignatureDeclareThrowsException")
	public void bind() throws Exception {

		GUI.getView().setTitle(I18N.tr("Settings"));
		final SettingsController control = new SettingsController(this);

		final SimpleContainer container = new SimpleContainer(getParent(), true, 1);

		final GridLayout gridLayout = new GridLayout(2, false);
		gridLayout.verticalSpacing = 10;

		final ColumnLayout columnLayout = new ColumnLayout(container.getComposite(), 2);
		final SimpleContainer left = new SimpleContainer(columnLayout.getComposite(), true);
		final SimpleContainer right = new SimpleContainer(columnLayout.getComposite(),false, 1);

		left.addHeadline(I18N.tr("Label Settings"));
		final SimpleContainer genSet = new SimpleContainer(left.getComposite(),
				false, 3);
		control.getLabelBackgroundButton().paint(genSet.getComposite());
		control.getBarColorButton().paint(genSet.getComposite());
		control.getBarVisible().paint(genSet.getComposite());

		left.addText("", false);
		left.addPart(control.getBarUpper());
		left.addHeadline(I18N.tr("Title Style"));
		left.addPart(control.getTitleStyle());
		left.addPart(control.getBarLower());
		left.addHeadline(I18N.tr("Content Style"));
		left.addPart(control.getContentStyle());
		left.addHeadline(I18N.tr("Footer Style"));
		left.addPart(control.getFooterStyle());
		left.addSeparator();
		final SimpleContainer idExample = new SimpleContainer(left.getComposite(), false,
				4);
		idExample.addLabelPair(control.getIdExample(), control.getIdVisible());
		control.getIdExample().getControl()
				.setVisible((boolean) control.getIdVisible().getValue());

		right.addHeadline(I18N.tr("Register Settings"));
		right.addText("", false);
		right.addHeadline(I18N.tr("Index"));
		right.addPart(control.getRegisterIndexStyle());
		right.addText("", false);
		right.addHeadline(I18N.tr("Description"));
		right.addPart(control.getRegisterContentStyle());

		final ButtonArea buttons = new ButtonArea();
		buttons.addButton(I18N.tr("Save"), new Action() {
			/**
			 * @param Object
			 */
			public void handleAction(final Object context)
					throws ApplicationException {
				control.speichern();
			}
		}, null, true, "document-save.png");
		buttons.addButton(I18N.tr("Cancel"), new ViewOrdnerVerwaltungAction());

		container.addText("  ", false);
		container.addButtonArea(buttons);

	}

	/**
	 * Liefert den Tab-Folder, in dem die einzelnen Module der Einstellungen
	 * untergebracht sind.
	 * 
	 * @return der Tab-Folder.
	 */
//	private TabFolder getTabFolder() {
//		if (this.folder != null)
//			return this.folder;
//
//		this.folder = new TabFolder(getParent(), SWT.NONE);
//		this.folder.setLayoutData(new GridData(GridData.FILL_BOTH));
//		return this.folder;
//	}

}
