/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;

/**
 * Klasse zum anzeigen ein Linie
 * 
 * @author munstfr
 * @version 1.0
 */
public class HorizontalLinePart implements Part {

	/**
	 * Minimum Size of the Line/Canvas
	 */
	private static final int MIN_SIZE = 2;

	/**
	 * Canvas zum Draufmalen
	 */
	private transient Canvas canvas;

	/**
	 * Die Farbe der Linie
	 */
	private Color color;

	/**
	 * die Gr&ouml;&szlig;e der Linie/Canvas
	 */
	private Point size;

	/**
	 * Soll eine Rahmen gezeichnt werden. Rahmenfarbe ist static Dunkelgrau.
	 */
	private Boolean border;

	/**
	 * Sichtbarkeit der Linie
	 */
	private boolean visible;

	/**
	 * Konstruktor mit Basisangaben
	 * 
	 * @param color
	 *            die Farbe der Linie
	 * @param size
	 *            die Gr&ouml;&szlig;e der Linie
	 * @param border
	 *            mit Rahmen (Standartfarbe Dunkelgrau)
	 */
	public HorizontalLinePart(final Color color, final Point size,
			final Boolean border) {

		this.color = color;
		this.size = size;
		this.border = border;
		this.visible = true;

	}

	/**
	 * @return the color
	 */
	public Color getColor() {
		return color;
	}

	/**
	 * @param color
	 *            the color to set
	 */
	public void setColor(final Color color) {
		this.color = color;
		redraw();
	}

	/**
	 * @return the size
	 */
	public Point getSize() {
		return size;
	}

	/**
	 * @param size
	 *            the size to set
	 */
	public void setSize(final Point size) {
		this.size = size;
		redraw();
	}

	/**
	 * @return the border
	 */
	public Boolean getBorder() {
		return border;
	}

	/**
	 * @param border
	 *            the border to set
	 */
	public void setBorder(final Boolean border) {
		this.border = border;
		redraw();
	}

	/**
	 * Setzt die Sichtbarkeit der Linie
	 * 
	 * @param visible
	 *            Linie Sichtbar?
	 */
	public void setVisible(final boolean visible) {
		this.visible = visible;
		redraw();
	}

	/**
	 * Liefert die Sichtbarkeit der Linie
	 * 
	 * @return Sichtbarkeit der Linie
	 */
	public boolean isVisible() {
		return this.visible;
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		if (canvas != null) {
			return; // NOPMD
		}

		final Composite container = new Composite(parent, SWT.NONE);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		container.setLayout(gridLayout);

		canvas = new Canvas(container, SWT.NONE);
		if (size != null) {
			final GridData gridData = new GridData(SWT.NONE);
			gridData.widthHint = size.x;
			gridData.heightHint = size.y;
			canvas.setSize(size);
			canvas.setLayoutData(gridData);
			canvas.layout();
			container.layout();
			canvas.setVisible(isVisible());
		}

		canvas.addPaintListener(new PaintListener() {

			/**
			 * @see org.eclipse.swt.events.PaintListener#paintControl(org.eclipse.swt.events.PaintEvent)
			 */
			@Override
			public void paintControl(final PaintEvent paintEvent) {

				if (color == null) {
					// Benutzte default Cyan als Linienafrbe
					paintEvent.gc.setBackground(GUI.getDisplay()
							.getSystemColor(SWT.COLOR_CYAN));
				} else {
					paintEvent.gc.setBackground(color);
				}
				paintEvent.gc.fillRectangle(calculateLineSize());
				if (border != null && border) {
					paintEvent.gc.setForeground(GUI.getDisplay()
							.getSystemColor(SWT.COLOR_DARK_GRAY));
					paintEvent.gc.drawRectangle(calculateLineSize());
				}
			}
		});

	}

	private void redraw() {
		if (canvas != null) {
			this.canvas.setVisible(isVisible());
			this.canvas.redraw();
		}
	}

	private Rectangle calculateLineSize() {

		final Rectangle calculatedSize = new Rectangle(1, 1, 0, 0);
		final Rectangle clientArea = canvas.getClientArea();
		if (this.size == null) {

			if (clientArea.width > MIN_SIZE) {
				calculatedSize.width = clientArea.width - MIN_SIZE;
			} else {
				calculatedSize.width = clientArea.width;
			}
			if (clientArea.height > MIN_SIZE) {
				calculatedSize.height = clientArea.height - MIN_SIZE;
			} else {
				calculatedSize.x = 0;
				calculatedSize.height = clientArea.height;
			}

		} else {

			if (clientArea.width < size.x) {
				calculatedSize.width = Math.max(clientArea.width - MIN_SIZE, 1);
			} else {
				calculatedSize.width = size.x;
			}

			if (clientArea.height < size.y) {
				calculatedSize.height = Math.max(clientArea.height - MIN_SIZE,
						1);
			} else {
				calculatedSize.height = size.y;
			}

		}

		if (calculatedSize.width <= MIN_SIZE) {
			calculatedSize.x = 0;
		}
		if (calculatedSize.height <= MIN_SIZE) {
			calculatedSize.y = 0;
		}

		return calculatedSize;
	}
}
