/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.util.TextUtils;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.TablePart;

/**
 * Part das alle gespeicherten Ordner in einer Tabelle anzeigt.
 * 
 * @author munstfr
 * @version 1.0
 */
public class OrdnerTablePart extends TablePart implements Part {

	/**
	 * Konstruktor zum Erstellen der Tabelle mit allen Ordnern.
	 * 
	 * @param action
	 *            Aktion beim Doppelklick
	 * @throws RemoteException
	 */
	public OrdnerTablePart(final Action action) throws RemoteException {
		this(init(), action);
	}

	/**
	 * Konstruktor zum Erstellen der Tabelle mit &uuml;bergebener Ordner Liste.
	 * 
	 * @param list
	 *            Liste der Ordner die angezeigt werden soll
	 * @param action
	 *            Aktion beim Doppelklick
	 */
	public OrdnerTablePart(final GenericIterator list, final Action action) {
		super(list, action);

		setRememberOrder(true);
		setRememberState(true);
		setMulti(true);

		addColumn("ID", "id", null, false);
		addColumn("Titel", "title", new LabelContent(20), false);
		addColumn("Inhalt", "content", new LabelContent(40), false);

	}

	/**
	 * Liefert eine Liste mit alle Ordner in der Datenbank, sortier nach ID.
	 * 
	 * @return eine GenericIterator-Object mit alle Ordner
	 * @throws RemoteException
	 */
	private static GenericIterator init() throws RemoteException {
		final OrVerwDBService service = (OrVerwDBService) Settings
				.getDBService();
		final DBIterator list = service.createList(Ordner.class);
		list.setOrder("ORDER BY id asc");
		return list;
	}

	/**
	 * Ein Formatter zum bearbeiten der Titeltexte. Zeilenumbr&uuml;che werden
	 * durch " \ " ersetzt, damit es in der Tabelle sch&ouml;ner aussieht.
	 * 
	 * @author munstfr
	 * @version 1.0
	 * 
	 */
	private class LabelContent implements Formatter {

		/**
		 * Maximale L&auml;nge des Textes
		 */
		private transient final int length;

		/**
		 * Konstruktoer zum erstellen des Formatters mit angabe der maximalen
		 * L&auml;nge des Textes.
		 * 
		 * @param length
		 *            die maximale L&auml;nge des Textes. Ist der Text
		 *            l&auml;nger, wird der Rest durch "..." ersetzt
		 */
		public LabelContent(final int length) {
			this.length = length;

		}

		/**
		 * @see de.willuhn.jameica.gui.formatter.Formatter#format(java.lang.Object)
		 */
		@Override
		@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
		public String format(final Object object) {

			String result = "";

			if (object instanceof String) {
				result = TextUtils.format((String) object, length);
			}
			return result;

		}

	}

}
