/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.OrVerwDBService;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.util.I18N;

/**
 * Part das eine Tabelle mit allen Registereintr%auml;gen eines Ordner anzeigt.
 * 
 * @author munstfr
 * @version 1.0
 */
public class RegisterTablePart extends TablePart implements Part {

	/**
	 * Unser &Uuml;bersetzer
	 */
	private final static I18N I18N = Settings.getI18n();

	/**
	 * Konstruktor zum Erstellen der Tabelle mit allen Regsitereintr&auml;gen in
	 * der Datenbank. Sollte eigentlich nide benutzt werden.
	 * 
	 * @param action
	 *            Aktion bei Doppelklick
	 * @throws RemoteException
	 */
	public RegisterTablePart(final Action action) throws RemoteException {
		this(init(), action);
	}

	/**
	 * Konstruktor zum Erstellen der Tabelle mit &Uuml;bergabe der Liste.
	 * 
	 * @param list
	 *            Liste mit Registereintr&auml;gen
	 * @param action
	 *            Aktion bei Doppelklick
	 */
	public RegisterTablePart(final GenericIterator list, final Action action) {
		super(list, action);

		setRememberColWidths(true);
		setRememberOrder(true);
		setRememberState(true);
		setEnabled(true);

		// Wichtig Tabellenfelder angeben, nicht die aus der korespondierenden
		// Klasse !!!
		// addColumn("ID", "id", null, false);
		addColumn(I18N.tr("Index"), "idx", null, true);
		addColumn(I18N.tr("Description"), "content", null, true);

	}

	/**
	 * Liefert eien Liste mit allen Registereintr&auml;gen in der Datenbank
	 * 
	 * @return GenericIterator-Object mit allen Registereintr&auml;gen
	 * @throws RemoteException
	 */
	private static GenericIterator init() throws RemoteException {
		final OrVerwDBService service = (OrVerwDBService) Settings
				.getDBService();
		final DBIterator list = service.createList(Register.class);
		list.setOrder("ORDER BY id asc");

		return list;
	}

}
