/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts.labelpart;

import java.rmi.RemoteException;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelButtonListener;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelGeneralButtons;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelInputButtons;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.trenner.BarTrenner;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * Das Part f&uuml;r den Ordnerr&uuml;cken.
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.TooManyFields" })
public final class LabelPart implements Part, LabelButtonListener,
		LabelInputTextChangeListener {

	/**
	 * Die H&ouml;he der oberen Linie
	 */
	public static final int TB_HEIGHT = 40;
	/**
	 * Das obere Margin der oberen Linie
	 */
	public static final int TB_TOP_MARGIN = 30;
	/**
	 * Die H&ouml;he der mittleren Linie
	 */
	public static final int MB_HEIGHT = 20;
	/**
	 * Das obere Margin der mittleren Linie
	 */
	public static final int MB_TOP_MARGIN = 10;
	/**
	 * Die Linienh&ouml;e
	 */
	public static final int BAR_HEIGHT = 6;

	/**
	 * Die default Titeltextausrichtung
	 */
	private static final int TITEL_ALIGNMENT = SWT.CENTER;

	/**
	 * Der Hauptcontainer f&uuml;r den Ordnerr&uuml;cken
	 */
	private transient SimpleContainer label;

	/**
	 * 
	 */
	private Color backgroundColor;

	/**
	 * Die obere Linie
	 */
	private final transient BarTrenner bar1;
	/**
	 * Die untere Linie
	 */
	private final transient BarTrenner bar2;
	/**
	 * Sichtbarkeit der Linien
	 */
	private boolean barVisible;

	/**
	 * Farbe der Linien
	 */
	private Color barColor;

	/**
	 * Eingabefeld f&uuml;r den Titeltext
	 */
	private final transient LabelInputText titelTextPart;

	/**
	 * Der Text des Titelfeldes
	 */
	private String titelText;

	/**
	 * Eingabefeld f&uuml;r den Hauptbereiches
	 */
	private final transient LabelInputText contentTextPart;

	/**
	 * Der Text des Hauptbereiches
	 */
	private String contentText;

	/**
	 * Eingabefeld f&uuml;r den Fu&szlig;nereich
	 */
	private final transient LabelInputText footerTextPart;
	/**
	 * Des des Fu&szlig;nereiches
	 */
	private String footerText;

	/**
	 * Label f&uuml;r die ID
	 */
	private transient CLabel idCLabel;
	/**
	 * Der Text der ID
	 */
	private String idText;
	/**
	 * Sichtbarkeit der ID.
	 */
	private boolean idVisible;

	/**
	 * Die Breite des Labels
	 */
	private int width;

	// private int marginLeftRight;

	/**
	 * Die Berechnete Gr&ouml;&szlig;e des Titeltextfeldes
	 */
	private transient Point titleSize;
	/**
	 * Die Berechnete Gr&ouml;&szlig;e des Haupttextfeldes
	 */
	private transient Point contentSize;
	/**
	 * Die Berechnete Gr&ouml;&szlig;e des Fu&szlig;textfeldes
	 */
	private transient Point footerSize;

	/**
	 * Die Listener fr einen Button-Click-Event
	 */
	private final transient Set<LabelPartListener> listeners = new CopyOnWriteArraySet<>();

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public void addListener(final LabelPartListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public void removeListener(final LabelPartListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten
	 * Buttons
	 * 
	 * @param methode
	 */
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	private void notifyListenersButtonClicked(final LabelPartButtonObject button) {
		for (final LabelPartListener listener : listeners) {
			listener.notifyButtonClicked(button);
		}
	}

	/**
	 * Benachrichtigen der registrierten Listeners das der Text ge&auml;ndert
	 * wurde.
	 * 
	 */
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	private void notifyListenersTextChanged() {
		for (final LabelPartListener listener : listeners) {
			listener.notifyLabelTextChanged();
		}
	}

	/**
	 * Erstellt eine neue Ordnerbeschriftung mit der angegeben Breite.
	 * 
	 * @param size
	 *            - Breite des Labels in Millimeter
	 * @param listener
	 *            - Listener fr Knpfe
	 * 
	 */
	public LabelPart(final LabelSizeEnum size, final LabelPartListener listener) {

		if (size == null) {
			this.width = LabelSizeEnum.getDefault().getWidth();
		} else {
			this.width = size.getWidth();
		}
		barVisible = true;
		idVisible = true;

		addListener(listener);

		calculateSizes();
		final Point topBarSize = new Point(titleSize.x, TB_HEIGHT + BAR_HEIGHT);
		bar1 = new BarTrenner(topBarSize, TB_TOP_MARGIN, BAR_HEIGHT,
				getBarColor(), getBackgroundColor());
		titelTextPart = new LabelInputText(titleSize, TITEL_ALIGNMENT);
		titelTextPart.addListener(this);
		final Point middleBarSize = new Point(titleSize.x, MB_HEIGHT
				+ BAR_HEIGHT);
		bar2 = new BarTrenner(middleBarSize, MB_TOP_MARGIN, BAR_HEIGHT,
				getBarColor(), getBackgroundColor());
		contentTextPart = new LabelInputText(contentSize);
		contentTextPart.addListener(this);
		footerTextPart = new LabelInputText(footerSize);
		footerTextPart.addListener(this);

	}

	/**
	 * Liefert den Text des Titelfeldes
	 * 
	 * @return Den Text des Titelfeldes
	 */
	public String getTitelText() {
		return titelTextPart.getText();
	}

	/**
	 * Setzt den Text des Titelfeldes
	 * 
	 * @param titel
	 *            der Text des Titelfeldes
	 */
	public void setTitelText(final String titel) {
		if (titel != null) {
			titelText = titel;
			if (titelTextPart != null) {
				titelTextPart.setText(titel);
			}
		}
	}

	/**
	 * Liefert den Text des Hauptfeldes
	 * 
	 * @return Den Text des Hauptfeldes
	 */
	public String getContentText() {
		return contentTextPart.getText();
	}

	/**
	 * Setzt den Text des Hauptfeldes
	 * 
	 * @param content
	 *            der Text des Hauptfeldes
	 */
	public void setContentText(final String content) {
		if (content != null) {
			contentText = content;
			if (contentTextPart != null) {
				contentTextPart.setText(content);
			}
		}
	}

	/**
	 * Liefert den Text des Fu&szlig;feldes
	 * 
	 * @return Den Text des Fu&szlig;feldes
	 */
	public String getFooterText() {
		return footerTextPart.getText();
	}

	/**
	 * Setzt den Text des Fu&szlig;feldes
	 * 
	 * @param footer
	 *            Der Text des Fu&szlig;feldes
	 */
	public void setFooterText(final String footer) {
		if (footer != null) {
			footerText = footer;
			if (footerTextPart != null) {
				footerTextPart.setText(footer);
			}
		}
	}

	/**
	 * Liefert die ID
	 * 
	 * @return Die ID
	 */
	public String getIdText() {
		return idText;
	}

	/**
	 * Setzt die ID
	 * 
	 * @param idText
	 *            Die ID
	 */
	public void setIdText(final String idText) {
		this.idText = idText;
		if (idCLabel != null) {
			fillText();
		}
	}

	/**
	 * Liefert die Sichtbarkeit der ID
	 * 
	 * @return Die Sichtbarkeit der ID
	 */
	public boolean isIdVisible() {
		return idVisible;
	}

	/**
	 * Setzt die Sichtbarkeit der ID
	 * 
	 * @param visible
	 *            die Sichtbarkeit der ID
	 */
	public void setIdVisible(final boolean visible) {
		idVisible = visible;
		if (idCLabel != null) {
			idCLabel.setVisible(visible);
		}
	}

	/**
	 * Liefert die Hintergrundfarbe des Labels. Wurde noch keine gesetzt wird
	 * sie auf RGB(255, 255, 224) gesetzt.
	 * 
	 * @return Die Hintergrundfarbe des Labels
	 */
	public Color getBackgroundColor() {
		if (backgroundColor == null) {
			backgroundColor = new Color(GUI.getDisplay(), 255, 255, 224);
		}
		return backgroundColor;
	}

	/**
	 * Setzt die Hintergrundfarbe des Labels.
	 * 
	 * @param color
	 *            die Hintergrundfarbe des Labels.
	 */
	public void setBackgroundColor(final Color color) {
		if (color != null) {
			backgroundColor = color;
			if (label != null) {
				setBackgrounds();
			}
		}
	}

	/**
	 * Liefert die Farbe der Linien. Wurde noch keine Farbe gesetzt wird sie auf
	 * RGB(0, 0, 240) gesetzt.
	 * 
	 * @return Die Farbe der Linien
	 */
	public Color getBarColor() {
		if (barColor != null) {
			barColor = new Color(GUI.getDisplay(), 0, 0, 240);
		}
		return barColor;
	}

	/**
	 * Setzt die Farbe der Linien
	 * 
	 * @param color
	 *            die Farbe der Linien
	 */
	public void setBarColor(final Color color) {
		if (color != null) {
			this.barColor = color;
			if (bar1 != null) {
				bar1.setBarColor(color);
			}
			if (bar2 != null) {
				bar2.setBarColor(color);
			}
		}
	}

	/**
	 * Liefert die Sichtbarkeit der Linien
	 * 
	 * @return Die Sichtbarkeit der Linien
	 */
	public Boolean getBarVisible() {
		return barVisible;
	}

	/**
	 * 
	 * Setzt die Sichtbarkeit der Linien
	 * 
	 * @param lineVisible
	 *            die Sichtbarkeit der Linien
	 */
	public void setBarVisible(final boolean lineVisible) {
		this.barVisible = lineVisible;
		if (bar1 != null) {
			bar1.setVisible(this.barVisible);
		}
		if (bar2 != null) {
			bar2.setVisible(this.barVisible);
		}
	}

	/**
	 * Liefert die Breite des Labels
	 * 
	 * @return die Breite des Labels
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * Setzt die Breite des Labels
	 * 
	 * @param width
	 *            Die Breite des Labels
	 */
	public void setWidth(final int width) {
		this.width = width;
	}

	/**
	 * Liefert die Textausrichtung des angegebenen Eingabefeldes.
	 * 
	 * @param section
	 *            das Eingabefeld
	 * @return Die Textausrichtung
	 */
	public int getAlignment(final LabelPartSectionsEnum section) {

		int alignment;

		if (section == null) {
			alignment = SWT.LEFT;
		} else {
			switch (section) {
			case CONTENT:
				alignment = contentTextPart.getTextAlignment();
				break;
			case FOOTER:
				alignment = footerTextPart.getTextAlignment();
				break;
			case TITLE:
				alignment = titelTextPart.getTextAlignment();
				break;
			default:
				alignment = SWT.LEFT;
				break;
			}
		}
		return alignment;
	}

	/**
	 * Setzt die Textausrichtung in einem angegeben Eingabefeld.
	 * 
	 * @param section
	 *            das Eingabefeld.
	 * @param alignment
	 *            die Textausrichtung
	 */
	public void setAlignment(final LabelPartSectionsEnum section,
			final int alignment) {

		if (section != null) {
			switch (section) {
			case CONTENT:
				contentTextPart.setTextAlignment(alignment);
				break;
			case FOOTER:
				footerTextPart.setTextAlignment(alignment);
				break;
			case TITLE:
				titelTextPart.setTextAlignment(alignment);
				break;
			default:
				break;
			}
		}
	}

	/**
	 * Liefert den Schrifttyp ein dem angegeben Eingabefeld.
	 * 
	 * @param section
	 *            das Eingabefeld
	 * @return Den Schrifttyp
	 */
	public Font getFont(final LabelPartSectionsEnum section) {

		Font font;

		if (section == null) {
			font = GUI.getShell().getFont();
		} else {
			switch (section) {
			case CONTENT:
				font = contentTextPart.getFont();
				break;
			case FOOTER:
				font = footerTextPart.getFont();
				break;
			case TITLE:
				font = titelTextPart.getFont();
				break;
			default:
				font = GUI.getShell().getFont();
				break;
			}
		}
		return font;
	}

	/**
	 * Setzt den Schrifttyp in einem angegeben Eingabefeld.
	 * 
	 * @param section
	 *            das Eingabefeld
	 * @param font
	 *            den Schrifttyp
	 */
	public void setFont(final LabelPartSectionsEnum section, final Font font) {

		if (section != null) {
			switch (section) {
			case CONTENT:
				contentTextPart.setFont(font);
				break;
			case FOOTER:
				footerTextPart.setFont(font);
				break;
			case TITLE:
				titelTextPart.setFont(font);
				break;
			default:
				break;
			}
		}
	}

	/**
	 * Liefert die Textfarbe in einem angegeben Eingabefeld.
	 * 
	 * @param section
	 *            das Eingabefeld
	 * @return Die Textfarbe
	 */
	public Color getForgroundColor(final LabelPartSectionsEnum section) {

		Color color;

		if (section == null) {
			color = GUI.getShell().getForeground();
		} else {
			switch (section) {
			case CONTENT:
				color = contentTextPart.getForeground();
				break;
			case FOOTER:
				color = footerTextPart.getForeground();
				break;
			case TITLE:
				color = titelTextPart.getForeground();
				break;
			default:
				color = GUI.getShell().getForeground();
				break;
			}
		} 
			
		return color;
	}

	/**
	 * Setzt die Textfarbe in einem angegeben Eingabefeld.
	 * 
	 * @param section
	 *            das Eingabefeld
	 * @param color
	 *            die Farbe
	 */
	public void setForegroundColor(final LabelPartSectionsEnum section,
			final Color color) {

		if (section != null) {
			switch (section) {
			case CONTENT:
				contentTextPart.setForeground(color);
				break;
			case FOOTER:
				footerTextPart.setForeground(color);
				break;
			case TITLE:
				titelTextPart.setForeground(color);
				break;
			default:
				break;
			}
		}

	}

	/**
	 * @param titelAlignment
	 */
	public void setTitleAlignment(final int titelAlignment) {
		titelTextPart.setTextAlignment(titelAlignment);
	}

	/**
	 * Setzt die Hintergrundfarbe aller Widgets
	 */
	private void setBackgrounds() {
		bar1.setBackground(getBackgroundColor());
		bar2.setBackground(getBackgroundColor());
		titelTextPart.setBackground(getBackgroundColor());
		contentTextPart.setBackground(getBackgroundColor());
		footerTextPart.setBackground(getBackgroundColor());
		idCLabel.setBackground(getBackgroundColor());
	}

	/**
	 * Berechnet die Gr&ouml;&szlig; der Eingabefelder
	 */
	private void calculateSizes() {

		int locationX;
		int locationY;
		final Point dpi = GUI.getDisplay().getDPI();

		final int marginLeftRight = (int) (5 * 0.039 * dpi.x);
		locationX = (int) (width * 0.039 * dpi.x);
		locationY = (int) (190 * 0.039 * dpi.y);
		new Point(locationX, locationY);

		locationX -= 2 * marginLeftRight;
		locationY = (int) (20 * 0.039 * dpi.y);
		titleSize = new Point(locationX, locationY);

		locationY = (int) (80 * 0.039 * dpi.y);
		contentSize = new Point(locationX, locationY);

		locationY = (int) (20 * 0.039 * dpi.y);
		footerSize = new Point(locationX, locationY);

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelButtonListener#notifyInputButtonClicked(de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum,
	 *      de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum)
	 */
	@Override
	public void notifyInputButtonClicked(final LabelPartSectionsEnum section,
			final LabelPartButtonsEnum button) {

		if (section != null) {
			Font font;
			Color color;

			switch (section) {
			case CONTENT:
				font = contentTextPart.getFont();
				color = contentTextPart.getForeground();
				break;
			case FOOTER:
				font = footerTextPart.getFont();
				color = footerTextPart.getForeground();
				break;
			case TITLE:
				font = titelTextPart.getFont();
				color = titelTextPart.getForeground();
				break;
			case GENERAL:
				font = null; //NOPMD
				switch (button) {
				case BAR_COLOR:
					color = getBarColor();
					break;
				case BACKGROUND_COLOR:
					color = getBackgroundColor();
					break;
				default:
					color = null; //NOPMD
					break;
				}
				break;
			default:
				font = new Font(GUI.getDisplay(), new FontData());
				color = new Color(GUI.getDisplay(), new RGB(0, 0, 0));
				break;
			}
			notifyListenersButtonClicked(new LabelPartButtonObject(section,
					button, font, color));

		}
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		label = new SimpleContainer(parent, false);
		paintPart();

	}

	/**
	 * Die eigentliche Zeichenroutine.
	 */
	private void paintPart() {

		final GridLayout gridLayout = new GridLayout();
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.numColumns = 5;
		label.getComposite().setLayout(gridLayout);

		// leere Spalte links
		final Label emptyLeft = new Label(label.getComposite(), SWT.NONE);
		emptyLeft.setText("");
		GridData gridData = new GridData();
		gridData.horizontalIndent = 0;
		gridData.verticalIndent = 0;
		gridData.verticalSpan = 6;
		gridData.verticalAlignment = GridData.FILL;
		gridData.widthHint = 30;
		emptyLeft.setLayoutData(gridData);

		bar1.paint(label.getComposite());
		new Label(label.getComposite(), SWT.NONE);
		// leere Spalte rechts (vorletzte)
		final Label emptyRight = new Label(label.getComposite(), SWT.NONE);
		emptyRight.setText("");
		gridData = new GridData();
		gridData.horizontalIndent = 0;
		gridData.verticalIndent = 0;
		gridData.verticalSpan = 6;
		gridData.verticalAlignment = GridData.FILL;
		gridData.widthHint = 30;
		emptyRight.setLayoutData(gridData);

		new Label(label.getComposite(), SWT.NONE);

		titelTextPart.paint(label.getComposite());
		titelTextPart.setText("Titel");
		new LabelInputButtons(label.getComposite(),
				LabelPartSectionsEnum.TITLE, this);
		// (new Label(label.getComposite(), SWT.NONE)).setText("Label");

		final LabelGeneralButtons generalButtons = new LabelGeneralButtons(
				label.getComposite(), LabelPartSectionsEnum.GENERAL, this);
		gridData = new GridData();
		gridData.horizontalIndent = 0;
		gridData.verticalIndent = 0;
		gridData.verticalSpan = 4;
		gridData.verticalAlignment = GridData.FILL;
		generalButtons.setLayoutData(gridData);

		bar2.paint(label.getComposite());
		new Label(label.getComposite(), SWT.NONE);

		contentTextPart.paint(label.getComposite());
		contentTextPart.setText("Inhalt1\nInhalt2");
		new LabelInputButtons(label.getComposite(),
				LabelPartSectionsEnum.CONTENT, this);

		footerTextPart.paint(label.getComposite());
		new LabelInputButtons(label.getComposite(),
				LabelPartSectionsEnum.FOOTER, this);

		idCLabel = new CLabel(label.getComposite(), SWT.BORDER);
		gridData = new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1);
		gridData.widthHint = 30;
		idCLabel.setLayoutData(gridData);
		idCLabel.setRightMargin(5);
		idCLabel.setAlignment(SWT.RIGHT);
		idCLabel.setVisible(isIdVisible());

		// Texte
		fillText();
		setBackgrounds();

		label.getComposite().pack();
		label.getComposite().layout();

		// scroll.update();
		// parent.pack();

	}

	/**
	 * Aktualisiert die Texte in den Eingabefeldern
	 */
	private void fillText() {

		if (titelTextPart != null) {
			titelTextPart.setText(titelText);
		}
		if (contentTextPart != null) {
			contentTextPart.setText(contentText);
		}
		if (footerTextPart != null) {
			footerTextPart.setText(footerText);
		}
		if (idCLabel != null) {
			idCLabel.setText(idText);
		}

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.parts.labelpart.LabelInputTextChangeListener#notifytextChanged()
	 */
	@Override
	public void notifytextChanged() {
		notifyListenersTextChanged();
	}

}
