/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts.labelpart;

/**
 * @author munstfr
 * 
 */
public enum LabelSizeEnum {
	/**
	 * 5cm Ordner
	 */
	SIZE_5CM("5cm", 37),
	/**
	 * 7cm Ordner
	 */
	SIZE_7CM("7cm", 50),
	/**
	 * 8cm Ordner
	 */
	SIZE_8CM("8cm", 60);

	/**
	 * Bezeichnung des Labels
	 */
	private String value;
	/**
	 * Breite des Label in mm
	 */
	private int width;

	private LabelSizeEnum(final String value, final int width) {
		this.value = value;
		this.width = width;
	}

	/**
	 * @return Die Bezeichnung des Labels.
	 */
	public String getValue() {
		return value;
	}

	/**
	 * @return Die Breite des Labels in Millimeter.
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @return das Sanndartlabel
	 */
	public static LabelSizeEnum getDefault() {
		return LabelSizeEnum.SIZE_8CM;
	}

	/**
	 * Liefert das LabelSize Enum zu einer angegebene Breite.
	 * 
	 * @param width
	 *            die Breite, nach der gesucht werden soll.
	 * @return Das passende LabelSizeEnum oder null.
	 */
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	public static LabelSizeEnum getLabelSize(final Integer width) {

		LabelSizeEnum result = null;
		if (width != null) {
			for (final LabelSizeEnum labelSize : LabelSizeEnum.values()) {
				if (labelSize.getWidth() == width) {
					result = labelSize;
					break;
				}
			}
		}
		return result;
	}
}
