/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts.labelpart.buttons;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum;
import de.willuhn.util.I18N;

/**
 * Erstellt eine Komponente mit Button zum Einstellen der
 * Labelhintergrundeinstellunugen. Dieses sind die Farbe des Hintergrundes,
 * Sichtbarkeit und Farbe der Trennlinien und die Sichtbarkeit der ID-Nummer.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class LabelGeneralButtons extends Composite implements SelectionListener {

	/**
	 * Unser &Uuml;bersetzter
	 */
	private final static I18N I18N = Settings.getI18n();
	/**
	 * Der Standartabstand zwischen den Button und zum Rand
	 * 
	 */
	private final static int MARGIN = 5;

	/**
	 * Sichtbarkeit der Linien
	 */
	private transient Button barVisibleButton;

	/**
	 * Farbe der Linien
	 */
	private transient Button barColorButton;

	/**
	 * Sichtbarkeit der ID
	 */
	private transient Button idVisibleButton;
	/**
	 * 
	 */
	private transient Button bgColorButton;

	/**
	 * Der Bereich zu dem die Buttons geh&ouml;ren. Sollte eigentlich immer
	 * LabelPartSectionsEnum.GENERAL sein
	 */
	private final transient LabelPartSectionsEnum section;

	/**
	 * Die Listener fr einen Button-Click-Event
	 */
	private transient final Set<LabelButtonListener> listeners = new CopyOnWriteArraySet<>();

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public final void addListener(final LabelButtonListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public final void removeListener(final LabelButtonListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrieretn Listeners mit bergabe des angeklickten
	 * Buttons
	 * 
	 * @param methode
	 */
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	public final void notifyListeners(final LabelPartButtonsEnum button) {
		for (final LabelButtonListener listener : listeners) {
			listener.notifyInputButtonClicked(section, button);
		}
	}

	/**
	 * Konstruktor zum erstellen der Buttons.
	 * 
	 * @param parent
	 *            das SWT-Eltern_Composite in dem die Buttons gemalt werden
	 * @param section
	 *            der Bereich zu dem die Buttons geh&ouml;ren.
	 *            (LabelPartSectionsEnum)
	 * @param listener
	 *            der Listener, der benachrichtigt wird, wenn ein Button
	 *            gedr&uuml;ckt wurde.
	 */
	public LabelGeneralButtons(final Composite parent,
			final LabelPartSectionsEnum section,
			final LabelButtonListener listener) {

		super(parent, SWT.NONE);
		this.addListener(listener);
		this.section = section;

		drawButtons(this);

		// Layout anpassen
		final GridData gridData = new GridData(SWT.LEFT, SWT.TOP, true, true,
				1, 1);
		this.setLayoutData(gridData);

	}

	/**
	 * Liefert den gesetzt Bereich. M&uuml;sste eigentlich immer
	 * LabelPartSectionsEnum.GENERAL sein.
	 * 
	 * @return den gesetzten LabelPartSectionsEnum Bereich
	 */
	public LabelPartSectionsEnum getSection() {
		return section;
	}

	private void drawButtons(final Composite parent) {

		final Composite container = new Composite(parent, SWT.BORDER);

		final RowLayout rowLayout = new RowLayout(SWT.VERTICAL);
		rowLayout.fill = true;
		rowLayout.spacing = 15;
		rowLayout.marginWidth = 20;
		rowLayout.marginTop = 10;
		rowLayout.marginBottom = 20;
		// rl.marginLeft = 50;

		container.setLayout(rowLayout);

		final Label hgCfgLabel = new Label(container, SWT.NONE);
		hgCfgLabel.setText(I18N.tr("Background-") + "\n     " + I18N.tr("Settings"));
		final RowData rowData = new RowData();
		rowData.height = 40;
		hgCfgLabel.setLayoutData(rowData);

		bgColorButton = new Button(container, SWT.PUSH | SWT.CENTER);
		bgColorButton.setText(I18N.tr("Background Color"));
		bgColorButton.setToolTipText(I18N
				.tr("Sets background color of the folder label"));
		bgColorButton.addSelectionListener(this);

		barVisibleButton = new Button(container, SWT.PUSH | SWT.CENTER);
		barVisibleButton.setText(I18N.tr("Lines Visible"));
		barVisibleButton.setToolTipText(I18N
				.tr("Show or hide lines on the folder label"));
		barVisibleButton.addSelectionListener(this);

		barColorButton = new Button(container, SWT.PUSH | SWT.CENTER);
		barColorButton.setText(I18N.tr("Line Color"));
		barColorButton.setToolTipText(I18N.tr("Set color of lines"));
		barColorButton.addSelectionListener(this);

		idVisibleButton = new Button(container, SWT.PUSH | SWT.CENTER);
		idVisibleButton.setText(I18N.tr("Show ID"));
		idVisibleButton.setToolTipText(I18N.tr("Is ID Visible"));
		idVisibleButton.addSelectionListener(this);

		container.pack();
		this.setSize(this.getSize().x + MARGIN, this.getSize().y + MARGIN);

	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	@Override
	public void widgetDefaultSelected(final SelectionEvent evt) {
		// Nothing to do here
	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	@Override
	public void widgetSelected(final SelectionEvent evt) {

		final Object source = evt.getSource();
		if (source instanceof Button) {
			final Button button = (Button) source;
			if (bgColorButton.equals(button)) {
				notifyListeners(LabelPartButtonsEnum.BACKGROUND_COLOR);
			} else if (barVisibleButton.equals(button)) {
				notifyListeners(LabelPartButtonsEnum.BAR_VISIBLE);
			} else if (barColorButton.equals(button)) {
				notifyListeners(LabelPartButtonsEnum.BAR_COLOR);
			} else if (idVisibleButton.equals(button)) {
				notifyListeners(LabelPartButtonsEnum.ID_VISIBLE);
			}
		}
	}

}
