/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts.labelpart.buttons;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum;
import de.willuhn.jameica.gui.util.SWTUtil;

/**
 * Die Buttons f&uuml;r die Farben und Schriftauswahl der Ordnerr&uuml;cken
 * Felder.
 * 
 * @author munstfr
 * @version 1.0
 */
public class LabelInputButtons extends Composite implements SelectionListener {

	/**
	 * Fixer Abstand der Buttons untereinander und zum Rand
	 */
	private final static int MARGIN = 5;
	/**
	 * Gr&ouml;&szlig;e des Schrifttauswahl-Buttons
	 */
	private final static Point FONT_BUTTON_SIZE = new Point(55, 25);
	/**
	 * Gr&ouml;&szlig;e des T-Buttons
	 */
	private final static Point ALIGN_BUTTON_SIZE = new Point(25, 25);

	/**
	 * Das zu dem Button geh&ouml;rige Eingabefeld
	 */
	final private transient LabelPartSectionsEnum section;
	/**
	 * Der Schrifttyp Button
	 */
	private transient Button textFontButton;
	/**
	 * Der Textfarben Button
	 */
	private transient Button textColorButton;
	/**
	 * Ausrichtung-Links Button
	 */
	private transient Button textAlignLeft;
	/**
	 * Ausrichtung-Mitte Button
	 */
	private transient Button textAlignCenter;
	/**
	 * Ausrichtung-Rechts Button
	 */
	private transient Button textAlignRight;

	/**
	 * Die Listener fr einen Button-Click-Event
	 */
	private final transient Set<LabelButtonListener> listeners = new CopyOnWriteArraySet<>();

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public final void addListener(final LabelButtonListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public final void removeListener(final LabelButtonListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrieretn Listeners mit bergabe des angeklickten
	 * Buttons
	 * 
	 * @param methode
	 */
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	public final void notifyListeners(final LabelPartButtonsEnum button) {
		for (final LabelButtonListener listener : listeners) {
			listener.notifyInputButtonClicked(section, button);
		}
	}

	/**
	 * Konstruktor mit Angabe des SWT-Eltern-Composits, des zugeh&ouml;rigen
	 * Eingabefeldes, und dem Listener.
	 * 
	 * @param parent SWT-Eltern-Composit
	 * @param section des zugeh&ouml;rigen Eingabefeldes als LabelPartSectionsEnum
	 * @param listener der LabelButtonListener
	 */
	public LabelInputButtons(final Composite parent,
			final LabelPartSectionsEnum section,
			final LabelButtonListener listener) {

		super(parent, SWT.NONE);
		this.addListener(listener);
		this.section = section;

		drawButtons();

		// Layout anpassen
		final GridData gridData = new GridData(SWT.LEFT, SWT.TOP, false, false,
				1, 1);
		gridData.heightHint = this.getSize().y;
		gridData.widthHint = this.getSize().x;
		this.setLayoutData(gridData);

	}

	/**
	 * Liefert die LabelPartSectionsEnum der Buttons.
	 * 
	 * @return LabelPartSectionsEnum das zugeh&ouml;rigen Eingabefelds als
	 *         LabelPartSectionsEnum
	 * 
	 */
	public LabelPartSectionsEnum getSection() {
		return section;
	}

	private void drawButtons() {

		textFontButton = new Button(this, SWT.PUSH | SWT.CENTER);
		textFontButton.setLocation(MARGIN, MARGIN);
		textFontButton.setText("Schrift");
		textFontButton.setSize(FONT_BUTTON_SIZE);
		textFontButton.addSelectionListener(this);

		textColorButton = new Button(this, SWT.PUSH | SWT.CENTER);
		textColorButton.setLocation(MARGIN + textFontButton.getSize().x
				+ MARGIN, MARGIN);
		textColorButton.setImage(SWTUtil.getImage("colors-icon.png"));
		textColorButton.setSize(ALIGN_BUTTON_SIZE);
		textColorButton.addSelectionListener(this);

		final int secondRowTop = MARGIN + textFontButton.getBounds().y
				+ textFontButton.getSize().y + MARGIN;

		textAlignLeft = new Button(this, SWT.PUSH | SWT.CENTER);
		textAlignLeft.setLocation(MARGIN, secondRowTop);
		textAlignLeft.setSize(ALIGN_BUTTON_SIZE);
		textAlignLeft.setImage(SWTUtil.getImage("align_left.png"));
		textAlignLeft.addSelectionListener(this);

		textAlignCenter = new Button(this, SWT.CENTER);
		textAlignCenter.setLocation(
				textAlignLeft.getBounds().x + textAlignLeft.getBounds().width
						+ MARGIN, secondRowTop);
		textAlignCenter.setSize(ALIGN_BUTTON_SIZE);
		textAlignCenter.setImage(SWTUtil.getImage("align_center.png"));
		textAlignCenter.addSelectionListener(this);

		textAlignRight = new Button(this, SWT.CENTER);
		textAlignRight.setLocation(textAlignCenter.getBounds().x
				+ textAlignCenter.getBounds().width + MARGIN, secondRowTop);
		textAlignRight.setSize(ALIGN_BUTTON_SIZE);
		textAlignRight.setImage(SWTUtil.getImage("align_right.png"));
		textAlignRight.addSelectionListener(this);

		this.pack();
		this.setSize(this.getSize().x + MARGIN, this.getSize().y + MARGIN);

	}

	/**
	 * @param evt
	 */
	@Override
	public void widgetDefaultSelected(final SelectionEvent evt) {
		// hier brauchen wir nix zu tun
	}

	/**
	 * Eventlistner wenn ein Widget ausgew&auml;hlt wurde.
	 * 
	 * @param evt
	 *            das SelectionEvent
	 */
	@Override
	public void widgetSelected(final SelectionEvent evt) {

		final Object source = evt.getSource();
		if (source instanceof Button) {
			final Button button = (Button) source;
			if (button.equals(textFontButton)) {
				// System.out.println("Font button pressed");
				notifyListeners(LabelPartButtonsEnum.FONT);
			} else if (button.equals(textColorButton)) {
				// System.out.println("Text color pressed");
				notifyListeners(LabelPartButtonsEnum.COLOR);
			} else if (button.equals(textAlignLeft)) {
				// System.out.println("Left button pressed");
				notifyListeners(LabelPartButtonsEnum.ALIGN_LEFT);
			} else if (button.equals(textAlignCenter)) {
				// System.out.println("Center button pressed");
				notifyListeners(LabelPartButtonsEnum.ALIGN_CENTER);
			} else if (button.equals(textAlignRight)) {
				// System.out.println("Right button pressed");
				notifyListeners(LabelPartButtonsEnum.ALIGN_RIGHT);
			}
		}
	}

}
