/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.parts.labelpart.trenner;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;

/**
 * Part das eine Linie auf dem OrdnerR&uuml;cken anzeigt
 * 
 * @author munstfr
 * @version 1.0
 */
public class BarTrenner implements Part {

	/**
	 * Die Gr&ouml;&szlig;e des Parts
	 */
	private transient Point size;

	/**
	 * Oberer Abstand zur Linie
	 */
	private transient int marginTop;

	/**
	 * H6ouml;he der Linie
	 */
	private transient int barHight;

	/**
	 * Farbe der Linie
	 */
	private transient Color barColor;

	/**
	 * Farbe desd Hintergrundes
	 */
	private transient Color background;

	/**
	 * Sichtbarkeit der Linie. Hintergrund ist immer Sichbar!
	 */
	private transient boolean visible;

	/**
	 * Der Container, in dem alles gemalt wird
	 */
	private transient Composite container;
	/**
	 * Ein Label als Linie. Damit ist das &Auml;ndner einfacher.
	 */
	private transient Label bar;

	/**
	 * Konstruktor mit Angaben zur Gr&ouml;&szlig;e und Abst&auml;nden. Es
	 * werden Standartfarbenm verwendet.
	 * 
	 * @param size
	 *            die Gr&ouml;&szlig;e des Containers
	 * @param marginTop
	 *            der obere Abstand zur Linie
	 * @param barHight
	 *            die H&ouml;he der Linie
	 * @throws NullPointerException
	 */
	public BarTrenner(final Point size, final int marginTop, final int barHight)
			throws NullPointerException {
		this(size, marginTop, barHight, null, null);
	}

	/**
	 * Konstruktor mit Angaben zur Gr&ouml;&szlig;e, Abst&auml;nden und Farben
	 * 
	 * @param size
	 *            die Gr&ouml;&szlig;e des Containers
	 * @param marginTop
	 *            der obere Abstand zur Linie
	 * @param barHight
	 *            die H&ouml;he der Linie
	 * @param barColor
	 *            die Farbe der Linien
	 * @param background
	 *            die Farbe des Hintergrundes
	 * @throws NullPointerException
	 */
	public BarTrenner(final Point size, final int marginTop,
			final int barHight, final Color barColor, final Color background)
			throws NullPointerException {
		this(size, marginTop, barHight, barColor, background, true);
	}

	/**
	 * Konstruktor mit Angaben zur Gr&ouml;&szlig;e, Abst&auml;nden und Farben
	 * 
	 * @param size
	 *            die Gr&ouml;&szlig;e des Containers
	 * @param marginTop
	 *            der obere Abstand zur Linie
	 * @param barHight
	 *            die H&ouml;he der Linie
	 * @param barColor
	 *            die Farbe der Linien
	 * @param background
	 *            die Farbe des Hintergrundes
	 * @param visible
	 *            die Sichbarkeit der Linie
	 * @throws NullPointerException
	 */
	public BarTrenner(final Point size, final int marginTop,
			final int barHight, final Color barColor, final Color background,
			final boolean visible) throws IllegalArgumentException {

		// null checks
		if (size == null) {
			throw new IllegalArgumentException("Size cannot be Null");
		}

		this.size = size;
		if (barHight > 0 && barHight < this.size.y) {
			this.barHight = barHight;
		} else {
			this.barHight = (int) (size.y / 2);
		}

		if (marginTop < this.size.y + barHight) {
			this.marginTop = marginTop;
		} else {
			this.marginTop = this.size.y - barHight;
		}

		if (barColor == null) {
			this.barColor = barColor;
		} else {
			this.barColor = new Color(GUI.getDisplay(), 0, 0, 240);
		}

		if (background == null) {
			this.background = background;
		} else {
			this.background = new Color(GUI.getDisplay(), 255, 255, 224);
		}

		this.visible = visible;

	}

	/**
	 * @param size
	 *            das zu setzende Objekt size
	 */
	@SuppressWarnings({"PMD.UnusedPrivateMethod","PMD.EmptyIfStmt"})
	private void setSize(final Point size) {
		if (size != null) {
			this.size = size;
			if (bar != null) {
				//TODO conatiner neu zeichen
			}
		}
	}

	/**
	 * @param size
	 *            das zu setzende Objekt size
	 */
	@SuppressWarnings({"PMD.UnusedPrivateMethod","PMD.EmptyIfStmt"})
	private void setMarginTop(final int marginTop) {
		if (marginTop < this.size.y + barHight) {
			this.marginTop = marginTop;
			if (bar != null) {
				// TODO Label neu zeichen
			}
		}
	}

	/**
	 * @param barHight
	 *            das zu setzende Objekt barHight
	 */
	@SuppressWarnings({"PMD.UnusedPrivateMethod","PMD.EmptyIfStmt"})
	private void setBarHight(final int barHight) {
		if (barHight > 0 && barHight < this.size.y) {
			this.barHight = barHight;
			if (bar != null) {
				// TODO Label neu zeichen
			}
		}
	}

	/**
	 * @param barColor
	 *            das zu setzende Objekt barColor
	 */
	public void setBarColor(final Color barColor) {
		if (barColor != null) {
			this.barColor = barColor;
			if (bar != null) {
				bar.setBackground(barColor);
			}
		}

	}

	/**
	 * @param background
	 *            das zu setzende Objekt background
	 */
	public void setBackground(final Color background) {
		if (background != null) {
			this.background = background;
			if (container != null) {
				container.setBackground(background);
			}
		}
	}

	/**
	 * Setzt die Sichbarbeit der Linie
	 * 
	 * @param visible die Sichbarbeit der Linie
	 */
	public void setVisible(final boolean visible) {
		this.visible = visible;
		if (bar != null) {
			bar.setVisible(visible);
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	public void paint(final Composite parent) {

		container = new Composite(parent, SWT.NONE);

		final RowLayout rowLayout = new RowLayout(SWT.VERTICAL);
		container.setLayout(rowLayout);
		// int marginTopBottom = (int) (size.y /2 ) - barHight;
		final int marginLeftRight = (int) (size.x * 0.05);
		rowLayout.center = true;
		rowLayout.marginTop = marginTop;
		rowLayout.marginLeft = marginLeftRight;

		final GridData gridData = new GridData();
		gridData.heightHint = size.y;
		gridData.widthHint = size.x;
		container.setLayoutData(gridData);
		container.setBackground(background);

		if (bar == null) {
			bar = new Label(container, SWT.NONE);
		}

		final RowData rowData = new RowData();
		rowData.height = barHight;
		rowData.width = size.x - (2 * marginLeftRight);
		bar.setLayoutData(rowData);

		bar.setBackground(barColor);
		bar.setVisible(visible);

	}

}
