//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//
package de.frankmuenster.jameica.orverw.parts.register;

import java.rmi.RemoteException;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.Settings;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.util.I18N;

/**
 * Setzt Farben und Schriften fr das Register.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class RegisterSettingsPart implements Part, SelectionListener {

	/**
	 * Unser &Uuml;bersetzter
	 */
	private static final I18N I18N = Settings.getI18n();

	/**
	 * Die max. L&auml;nge des Beschreibungsfeldes
	 */
	private static final int CONTENT_LENGTH = 250;

	/**
	 * Die Hintergrundfarbe des Index
	 */
	private Color indexBgColor;

	/**
	 * Der Schrifttype des Index
	 */
	private Font indexFont;

	/**
	 * Die Hintergrundfarbe der Beschreibung
	 */
	private Color contentBgColor;

	/**
	 * Der Texttype der Beschreibung
	 */
	private Font contentFont;

	/**
	 * Der Container in dem alles gezeichnet wird
	 */
	private transient Composite container;
	/**
	 * Das Indexfeld
	 */
	private transient CLabel indexLabel;
	/**
	 * Das Beschreibungsfeld
	 */
	private transient CLabel contentLabel;

	/**
	 * Der Button f&uuml;r den Index-Schrifttyp
	 */
	private transient Button indexFontButton;
	/**
	 * Der Button f&uuml;r die Index-Hintergrundfarbe
	 */
	private transient Button indexBgColorBtn;
	/**
	 * Der Button f&uuml;r deb Beschreibungstexttyp
	 */
	private transient Button contentFontButton;
	/**
	 * Der Button f&uuml;r die Beschreibungstext-Hintergrundfarbe
	 */
	private transient Button contentBgColorBtn;

	/**
	 * Die Listener fr einen Button-Click-Event
	 */
	private final transient Set<RegisterSettingsListener> listeners = new CopyOnWriteArraySet<>();

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public final void addListener(final RegisterSettingsListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public final void removeListener(final RegisterSettingsListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten
	 * Buttons
	 * 
	 * @param buttonObject
	 *            mit Button, Schrift- und/oder Farbinformationen
	 */
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	public final void notifyListeners(
			final RegisterSettingsButtonObject buttonObject) {

		for (final RegisterSettingsListener listener : listeners) {
			listener.notifyButtonClicked(buttonObject);
		}
	}

	/**
	 * Erstellt ein neues RegisterSettingsPart Object
	 * 
	 * @param listener
	 *            RegisterSettingsListener
	 */
	public RegisterSettingsPart(final RegisterSettingsListener listener) {
		this.addListener(listener);
	}

	/**
	 * Liefert die Schrift des Index
	 * 
	 * @return die Schrift des Index
	 */
	public Font getIndexFont() {
		if (indexFont == null) {
			indexFont = GUI.getDisplay().getSystemFont();
		}
		return indexFont;
	}

	/**
	 * Setzt die Schrift des Indexes
	 * 
	 * @param font
	 *            zu setzende Schrift
	 */
	public void setIndexFont(final Font font) {
		if (font != null) {
			indexFont = font;
			if (indexLabel != null) {
				indexLabel.setFont(font);
				updateControls();
			}
		}
	}

	/**
	 * Liefert die Hintergrundfarbe des Index
	 * 
	 * @return die Hintergrundfarbe des Index
	 */
	public Color getIndexBgColor() {
		if (indexBgColor == null) {
			indexBgColor = new Color(GUI.getDisplay(), 255, 250, 150);
		}
		return indexBgColor;

	}

	/**
	 * Setzt die Hintergrundfarbe des Index
	 * 
	 * @param color
	 *            zu setzende Farbe
	 */
	public void setIndexBgColor(final Color color) {
		if (color != null) {
			indexBgColor = color;
			if (indexLabel != null) {
				indexLabel.setBackground(getLabelBg(indexLabel.getSize().x,
						indexLabel.getSize().y, color));
			}
		}
	}

	/**
	 * Liefert die Schrift der Beschreibung
	 * 
	 * @return die Schrift der Beschreibung
	 */
	public Font getContentFont() {
		if (contentFont == null) {
			contentFont = GUI.getDisplay().getSystemFont();
		}
		return contentFont;
	}

	/**
	 * Setzt die Schrift der Beschreibung
	 * 
	 * @param font
	 *            zu setzende Schrift
	 */
	public void setContentFont(final Font font) {
		if (font != null) {
			contentFont = font;
			if (contentLabel != null) {
				contentLabel.setFont(font);
				updateControls();
			}
		}
	}

	/**
	 * Liefert die Hintergrundfarbe der Beschreibung
	 * 
	 * @return die Hintergrundfarbe der Beschreibung
	 */
	public Color getContentBgColor() {
		if (contentBgColor == null) {
			contentBgColor = new Color(GUI.getDisplay(), 240, 240, 240);
		}
		return contentBgColor;

	}

	/**
	 * Setzt die Hintergrundfarbe der Beschreibung
	 * 
	 * @param color
	 *            zu setzende Farbe
	 */
	public void setContentBgColor(final Color color) {
		if (color != null) {
			contentBgColor = color;
			if (contentLabel != null) {
				contentLabel.setBackground(getLabelBg(contentLabel.getSize().x,
						contentLabel.getSize().y, color));
			}
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		if (container != null && !container.isDisposed()) {
			container.dispose();
		}

		container = new Composite(parent, SWT.NONE);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 7;
		container.setLayout(gridLayout);

		updateControls();
		container.pack();

	}

	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	private void updateControls() {

		for (final Control child : container.getChildren()) {
			child.dispose();
		}

		final int rowHight = getMinRowHight();

		indexLabel = new CLabel(container, SWT.NONE);
		indexLabel.setFont(getIndexFont());
		GridData gridData = new GridData(GridData.VERTICAL_ALIGN_CENTER);
		gridData.widthHint = getMinLabelWidth(indexLabel);
		gridData.heightHint = rowHight;
		indexLabel.setLayoutData(gridData);

		indexLabel.setText(I18N.tr("Index"));
		indexLabel.setLeftMargin(15);
		indexLabel.setBackgroundImage(getLabelBg(gridData.widthHint, rowHight,
				getIndexBgColor()));

		indexFontButton = new Button(container, SWT.PUSH | SWT.CENTER);
		indexFontButton.setText(I18N.tr("Font"));
		indexFontButton.setToolTipText(I18N.tr("Sets font style of the index"));
		indexFontButton.addSelectionListener(this);

		indexBgColorBtn = new Button(container, SWT.PUSH | SWT.CENTER);
		indexBgColorBtn.setImage(SWTUtil.getImage("colors-icon.png"));
		indexBgColorBtn.setToolTipText(I18N
				.tr("Sets background color of the index"));
		indexBgColorBtn.addSelectionListener(this);

		final Label leerLabel = new Label(container, SWT.NONE);
		leerLabel.setText("        ");

		contentLabel = new CLabel(container, SWT.NONE);
		gridData = new GridData(GridData.VERTICAL_ALIGN_CENTER);
		gridData.widthHint = CONTENT_LENGTH;
		gridData.heightHint = rowHight;
		contentLabel.setLayoutData(gridData);
		contentLabel.setText(I18N.tr("Description"));
		contentLabel.setLeftMargin(15);
		contentLabel.setFont(getContentFont());
		contentLabel.setBackgroundImage(getLabelBg(CONTENT_LENGTH, rowHight,
				getContentBgColor()));

		contentFontButton = new Button(container, SWT.PUSH | SWT.CENTER);
		contentFontButton.setText(I18N.tr("Font"));
		contentFontButton.setToolTipText(I18N
				.tr("Sets font style of the description"));
		contentFontButton.addSelectionListener(this);

		contentBgColorBtn = new Button(container, SWT.PUSH | SWT.CENTER);
		contentBgColorBtn.setImage(SWTUtil.getImage("colors-icon.png"));
		contentBgColorBtn.setToolTipText(I18N
				.tr("Sets background color of the description"));
		contentBgColorBtn.addSelectionListener(this);

		container.pack();
		container.layout();
		container.getParent().layout();
		// Layout new malen (ganz schn weit oben :-) )
		container.getParent().getParent().getParent().layout();

	}

	private int getMinLabelWidth(final CLabel label) {

		final GC myGc = new GC(label);

		final FontMetrics fontMetrics = myGc.getFontMetrics();
		final int charWidth = fontMetrics.getAverageCharWidth();
		myGc.dispose();
		return charWidth * 8 + 15;

	}

	private int getMinRowHight() {

		final int rowHightIndex = getIndexFont().getFontData()[0].getHeight();
		final int rowHightContent = getContentFont().getFontData()[0]
				.getHeight();
		final int hight = Math.max(rowHightIndex, rowHightContent);

		return (int) (hight * 1.8);

	}

	private Image getLabelBg(final int length, final int height,
			final Color color) {

		final Image image = new Image(GUI.getDisplay(), length, height);

		final GC myGc = new GC(image);
		myGc.setBackground(color);
		myGc.fillOval(0, 0, height / 2, height);
		myGc.fillRectangle(height / 4, 0, length, height);
		myGc.dispose();

		return image;
	}

	/**
	 * 
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse
	 *      .swt.events.SelectionEvent)
	 */
	@Override
	public void widgetDefaultSelected(final SelectionEvent arg0) {
		// hier nix zu tun
	}

	/**
	 * 
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt
	 *      .events.SelectionEvent)
	 */
	@Override
	public void widgetSelected(final SelectionEvent evt) {

		final Object source = evt.getSource();
		if (source instanceof Button) {
			final Button button = (Button) source;
			if (indexFontButton.equals(button)) {
				final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
						RegisterSettingsButtonsEnum.INDEX_FONT, indexFont);
				notifyListeners(btnObj);
			} else if (indexBgColorBtn.equals(button)) {
				final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
						RegisterSettingsButtonsEnum.INDEX_BACKGROUND_COLOR,
						indexBgColor);
				notifyListeners(btnObj);
			} else if (contentFontButton.equals(button)) {
				final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
						RegisterSettingsButtonsEnum.CONTENT_FONT, contentFont);
				notifyListeners(btnObj);
			} else if (contentBgColorBtn.equals(button)) {
				final RegisterSettingsButtonObject btnObj = new RegisterSettingsButtonObject(
						RegisterSettingsButtonsEnum.CONTENT_BACKGROUND_COLOR,
						contentBgColor);
				notifyListeners(btnObj);
			}
		}
	}

}
