/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.print;

import net.sf.paperclips.PageNumberPageDecoration;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.Print;
import net.sf.paperclips.SimplePageDecoration;
import net.sf.paperclips.TextPrint;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;

import de.frankmuenster.jameica.orverw.Settings;
import de.willuhn.jameica.gui.util.Font;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Abstrakte Klasse die ein Print-Interface bereit stellt.
 * 
 * @author munstfr
 * @version 1.0
 */
public abstract class AbstractPrint {
	
	/**
	 * Unser &Uuml;bersetzer
	 */
	protected static final I18N I18N = Settings.getI18n();

	/**
	 * Der Schrifttyp des Titelfeldes 
	 */
	protected static final FontData FONT_BOLD = Font.BOLD.getSWTFont().getFontData()[0];
	/**
	 * Die System-Standartschrift
	 */
	protected static final FontData FONT_NORMAL = Font.SMALL.getSWTFont().getFontData()[0];
	/**
	 * Die System-Standartschrift in FETT
	 */
	protected static final FontData FONT_NORMAL_BOLD = new FontData(FONT_NORMAL.getName(),
			FONT_NORMAL.getHeight(), SWT.BOLD);

	/**
	 * Die System-Standartschrift in KLEIN
	 */
	protected static final FontData FONT_TINY = new FontData(FONT_NORMAL.getName(),
			FONT_NORMAL.getHeight() - 2, SWT.NORMAL);
	
	/**
	 *  Die System-Standartschrift in KLEIN und FETT
	 */
	protected static final FontData FONT_TINY_BOLD = new FontData(FONT_TINY.getName(),
			FONT_TINY.getHeight(), SWT.BOLD);

	/**
	 * Standartkonstruktor 
	 */
	@SuppressWarnings("PMD.UnnecessaryConstructor")
	public AbstractPrint() {
		// intentionally left blank
	}

	/**
	 * Liefert den Druckinhalt als Print-Objekt
	 * @return Print mit Druckinhalt
	 * @throws ApplicationException 
	 */
	protected abstract Print getPrintContent() throws ApplicationException;

	/**
	 * Liefert den Titel des Drucks als String
	 * @return Der Titel des Drucks als String
	 */
	public abstract String getTitle();

	/**
	 * Erstellt und liefert das Print-Objekt
	 * @return Das Print-Objekt
	 * @throws ApplicationException
	 */
	public Print createPrint() throws ApplicationException {

		final Print content = getPrintContent();
		final PagePrint page = new PagePrint(content);

		page.setHeader(new SimplePageDecoration(new TextPrint(getTitle(),
				SWT.CENTER)));
		page.setHeaderGap(5);
		page.setFooterGap(5);

		final PageNumberPageDecoration footer = new PageNumberPageDecoration(
				SWT.RIGHT);
		footer.setFontData(FONT_TINY);
		page.setFooter(footer);

		// Falls eine Sub-Klasse noch was ndern will, jetzt gilt's
		this.customize(page);

		return page;
	}

	/**
	 * Hier kann eine Sub-Klasse das Print-Objekt nochmal &auml;ndern.
	 * 
	 * @param page Der ganze Ausdruck als PrintPage-Objekt
	 * @throws ApplicationException
	 */
	protected abstract void customize(PagePrint page) throws ApplicationException;

}
