/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;

import net.sf.paperclips.BorderPrint;
import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.EmptyPrint;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.LayerPrint;
import net.sf.paperclips.LineBorder;
import net.sf.paperclips.LinePrint;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.Print;
import net.sf.paperclips.TextPrint;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.RGB;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelSizeEnum;
import de.frankmuenster.jameica.orverw.util.TextUtils;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
@SuppressWarnings("PMD.BeanMembersShouldSerialize")
public class PrintLabel extends AbstractPrint {

	/**
	 * Der Ordner dessen Label gedruckt werden soll
	 */
	final private Ordner ordner;
	/**
	 * Der Titel
	 */
	final private String titel;
	/**
	 * Die Gre des Labels
	 */
	final private LabelSizeEnum size;
	/**
	 * die Hintergrundfarbe
	 */
	final private RGB lblBgColor;

	/**
	 * Der Konstruktor
	 * 
	 * @param ordner
	 *            der Ordner dessen Label gedruckt werden soll
	 * @throws ApplicationException
	 */
	public PrintLabel(final Ordner ordner) throws ApplicationException {

		super();

		this.ordner = ordner;
		try {
			this.size = LabelSizeEnum.getLabelSize(ordner.getWidth());
			this.lblBgColor = ordner.getLabelBackground().getRGB();
			this.titel = ordner.getTitel();
		} catch (RemoteException e) {
			throw new ApplicationException(
					I18N.tr("Error instanciating print object"), e);
		}
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#getPrintContent()
	 */
	@Override
	protected Print getPrintContent() throws ApplicationException {

		final DefaultGridLook look = new DefaultGridLook();
		look.setBodyBackground(lblBgColor);
		look.setCellPadding(4, 0);
		look.setHeaderBackground(new RGB(200, 200, 200));

		final int width = (int) ((size.getWidth() - 8) * 0.039 * 72);
		final int topMargin = (int) (5 * 0.039 * 72);
		// int bar = (int) (4 * 0.039 * 72);
		final int titleMargin = (int) (20 * 0.039 * 72);
		final int contentMargin = (int) (80 * 0.039 * 72);
		// int footer = (int) (80 * 0.039 * 72);
		// int id = (int) (80 * 0.039 * 72);

		final GridPrint outer = new GridPrint(String.valueOf(width + 10));
		final GridPrint grid = new GridPrint(String.valueOf(width), look);

		try {
			final EmptyPrint topP = new EmptyPrint(width, topMargin);
			final LinePrint barP1 = new LinePrint(SWT.HORIZONTAL, (float) 4.0);
			barP1.setRGB(ordner.getBarColor().getRGB());

			// Title
			final EmptyPrint titleEmpty = new EmptyPrint(width, titleMargin);
			final TextPrint titleTextPrint = new TextPrint(ordner.getTitel(),
					ordner.getTitelFont().getFontData()[0],
					ordner.getTitelAlignment());
			titleTextPrint.setForeground(ordner.getTitelColor().getRGB());
			final LayerPrint titleLayerPrint = new LayerPrint();
			titleLayerPrint.add(titleEmpty);
			titleLayerPrint.add(titleTextPrint, ordner.getTitelAlignment());

			// Content
			final EmptyPrint contentEmpty = new EmptyPrint(width, contentMargin);
			final TextPrint contentTextPrint = new TextPrint(
					ordner.getContent(),
					ordner.getContentFont().getFontData()[0],
					ordner.getContentAlignment());
			contentTextPrint.setForeground(ordner.getContentColor().getRGB());
			final LayerPrint contentLayerPrint = new LayerPrint();
			contentLayerPrint.add(contentEmpty);
			contentLayerPrint.add(contentTextPrint,
					ordner.getContentAlignment());

			// Footer
			final EmptyPrint footerEmpty = new EmptyPrint(width, titleMargin);
			final TextPrint footerTextPrint = new TextPrint(ordner.getFooter(),
					ordner.getFooterFont().getFontData()[0],
					ordner.getFooterAlignment());
			footerTextPrint.setForeground(ordner.getFooterColor().getRGB());
			final LayerPrint footerLayerPrint = new LayerPrint();
			footerLayerPrint.add(footerEmpty);
			footerLayerPrint.add(footerTextPrint, ordner.getFooterAlignment());

			final LinePrint barP2 = new LinePrint(SWT.HORIZONTAL, (float) 4.0);
			barP2.setRGB(ordner.getBarColor().getRGB());

			// grid.add(gridBorderPrint);
			grid.add(topP);
			grid.add(barP1);
			grid.add(titleLayerPrint);
			grid.add(barP2);
			grid.add(contentLayerPrint);
			grid.add(footerLayerPrint);

			final LineBorder lineBorder = new LineBorder();
			lineBorder.setGapSize(1);
			lineBorder.setLineWidth(1);
			final BorderPrint gridBorderPrint = new BorderPrint(grid,
					lineBorder);
			outer.add(gridBorderPrint);

		} catch (RemoteException e) {
			throw new ApplicationException(I18N.tr(
					"Error creating label print for folder {}", titel), e);
		}
		return outer;
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.print.AbstractPrintSupport#getTitle()
	 */
	public String getTitle() {
		return TextUtils.format(titel, 10);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#customize(net.sf.paperclips.PagePrint)
	 */
	@Override
	protected void customize(final PagePrint page) throws ApplicationException {
		// hier brauchen wir nix tun tun
	}

}
