/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import net.sf.paperclips.DefaultGridLook;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.ImagePrint;
import net.sf.paperclips.LayerPrint;
import net.sf.paperclips.PageDecoration;
import net.sf.paperclips.PageNumber;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.Print;
import net.sf.paperclips.TextPrint;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.util.TextUtils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.util.ApplicationException;

/**
 * Implementiert den Ausdruck des Registers.
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings("PMD.BeanMembersShouldSerialize")
public class PrintRegister extends AbstractPrint {

	/**
	 * Der Registerinhalt
	 */
	final private DBIterator index;
	/**
	 * Der Ordner Titel
	 */
	final private String title;
	/**
	 * Die Ordner ID
	 */
	final private String ordnerID;
	/**
	 * Das letzte Speicherdatum
	 */
	final private Date lastSaveDate;

	/**
	 * Der Titeltext Schrifttyp
	 */
	final private Font titleFont;
	/**
	 * der Indextext Schrifttyp
	 */
	final private Font indexFont;
	/**
	 * Der Beschreibungstext Schrifttyp
	 */
	final private Font contentFont;

	/**
	 * Die Hintergrundfarbe des Indexfeldes
	 */
	final private Color indexBgColor;
	/**
	 * Die Hintergrundfarbe des Beschreibungsfeldes
	 */
	final private Color contentBgColor;

	/**
	 * Der Konstruktor
	 * 
	 * @param ordner
	 *            der Ordner dessen Register gedruckt werden soll.
	 * @throws ApplicationException
	 */
	public PrintRegister(final Ordner ordner) throws ApplicationException {

		super();

		try {

			index = ordner.getInhaltsverzeichniss();
			title = ordner.getTitel();
			ordnerID = ordner.getID();
			lastSaveDate = ordner.getDate();

			titleFont = ordner.getTitelFont();
			indexFont = ordner.getRegisterIndexFont();
			contentFont = ordner.getRegisterContentFont();

			indexBgColor = ordner.getRegisterIndexBackground();
			contentBgColor = ordner.getRegisterContentBackground();

		} catch (RemoteException e) {
			throw new ApplicationException(
					I18N.tr("Error instaniating register print object"), e);
		}

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#getPrintContent()
	 */
	@Override
	@SuppressWarnings({ "PMD.AvoidInstantiatingObjectsInLoops",
			"PMD.DataflowAnomalyAnalysis" })
	protected Print getPrintContent() throws ApplicationException {

		final DefaultGridLook look = new DefaultGridLook();
		look.setCellSpacing(5, 5);
		final GridPrint grid = new GridPrint("50, r:d", look);

		// Kopfzeile
		final TextPrint headerRow = new TextPrint(this.title,
				titleFont.getFontData()[0]);
		grid.addHeader(SWT.LEFT, headerRow, 2);

		// Leere Zeile
		final TextPrint leer = new TextPrint(" ", FONT_BOLD);
		grid.add(SWT.LEFT, leer, 2);

		final double factor = (double) GUI.getDisplay().getDPI().x
				/ (double) 72;
		final int indexFontHeight = (int) (indexFont.getFontData()[0]
				.getHeight() * factor);
		final int regFontHeight = (int) (contentFont.getFontData()[0]
				.getHeight() * factor);
		int height = Math.max(indexFontHeight, regFontHeight);
		height += 8;
		final int indexFontX = (height - indexFontHeight) / 2 - 1;
		final int regsiterFontX = (height - regFontHeight) / 2 - 1;
		try {
			while (index.hasNext()) {
				final Register register = (Register) index.next();

				LayerPrint layerPrint = new LayerPrint();
				ImageData imageData = getImage(height, 40, indexBgColor);
				ImagePrint imagePrint = new ImagePrint(imageData, new Point(72,
						72));
				DefaultGridLook look1 = new DefaultGridLook();
				look1.setCellSpacing(0, 0);
				look1.setCellPadding(10, indexFontX);
				GridPrint text = new GridPrint("r:d", look1);
				TextPrint textPrint = new TextPrint(register.getIndex(),
						indexFont.getFontData()[0]);
				text.add(textPrint);

				layerPrint.add(imagePrint);
				layerPrint.add(text);
				grid.add(layerPrint);

				layerPrint = new LayerPrint();
				imageData = getImage(height, 200, contentBgColor);
				imagePrint = new ImagePrint(imageData, new Point(72, 72));
				look1 = new DefaultGridLook();
				look1.setCellSpacing(0, 0);
				look1.setCellPadding(15, regsiterFontX);
				text = new GridPrint("r:d", look1);
				textPrint = new TextPrint(register.getBeschreibung(),
						contentFont.getFontData()[0]);
				text.add(textPrint);

				layerPrint.add(imagePrint);
				layerPrint.add(text);
				grid.add(layerPrint);

			}

		} catch (RemoteException e) {
			throw new ApplicationException(I18N.tr(
					"Error during creating register print for folder {0}",
					title), e);
		}

		return grid;

	}

	/**
	 * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#getTitle()
	 */
	@Override
	public String getTitle() {
		return TextUtils.format(title, 10);
	}

	private ImageData getImage(final int height, final int width,
			final Color color) {

		final Image image = new Image(GUI.getDisplay(), width, height);

		@SuppressWarnings("PMD.ShortVariable")
		final GC gc = new GC(image);
		gc.setBackground(color);
		gc.fillOval(0, 0, height / 2, height);
		gc.fillRectangle(height / 4, 0, width, height);
		gc.dispose();

		return image.getImageData();

	}

	private String getLastSaveDate() {
		final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.YYYY",
				Locale.GERMAN);
		return sdf.format(lastSaveDate);
	}

	/**
	 * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#customize(net.sf.
	 *      paperclips.PagePrint)
	 */
	@Override
	@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
	protected void customize(final PagePrint page) throws ApplicationException {

		// Neuer Header mit ID
		final PageDecoration header = new PageDecoration() {

			/**
			 * @see net.sf.paperclips.PageDecoration#createPrint(net.sf.paperclips.PageNumber)
			 */
			@Override
			public Print createPrint(final PageNumber pageNumber) {

				GridPrint header = new GridPrint("d:g,r:d");
				TextPrint title1 = new TextPrint("", FONT_TINY, SWT.LEFT);
				TextPrint title2 = new TextPrint("ID " + ordnerID, FONT_TINY,
						SWT.RIGHT);
				header.add(title1);
				header.add(title2);

				return header;
			}
		};
		page.setHeader(header);

		// neuer Foot mit Datum und Seitenangeben
		final PageDecoration footer = new PageDecoration() {

			/**
			 * net.sf.paperclips.PageDecoration#createPrint(net.sf.paperclips
			 * .PageNumber)
			 */
			@Override
			public Print createPrint(final PageNumber pageNumber) {

				GridPrint footer = new GridPrint("d:g, r:d");
				TextPrint datum = new TextPrint(getLastSaveDate(), SWT.LEFT);
				datum.setFontData(FONT_TINY);
				TextPrint seite = new TextPrint(
						(pageNumber.getPageNumber() + 1) + "/"
								+ (pageNumber.getPageCount()), SWT.RIGHT);
				seite.setFontData(FONT_TINY);

				footer.add(datum);
				footer.add(seite);

				return footer;
			}
		};
		page.setFooter(footer);
	}
}
