/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;

import net.sf.paperclips.EmptyPrint;
import net.sf.paperclips.GridPrint;
import net.sf.paperclips.Margins;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.PaperClips;
import net.sf.paperclips.Print;
import net.sf.paperclips.PrintJob;
import net.sf.paperclips.ui.PrintViewer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseWheelListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.printing.PrintDialog;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.frankmuenster.jameica.orverw.Settings;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ScrolledContainer;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.I18N;

/**
 * Zeigt eine einfache Druckvorschau. Ist noch verbesserungswrdig. Benutzt das
 * PrintViewer Objekt aus paperclips, da PrintPreview in Linux noch buggy ist.
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings({ "PMD.BeanMembersShouldSerialize", "PMD.ExcessiveImports" })
public class PrintViewDialog {

	/**
	 * Unser &&Uuml;bersetzer
	 */
	private static final I18N I18N = Settings.getI18n();

	/**
	 * Die Shell fr den Dialog
	 */
	private Shell shell;

	/**
	 * Ein Scrolled Container, damit auch alles Sichtbar ist
	 */
	private ScrolledContainer scroll;
	/**
	 * Das PrintViewer Objekt.
	 */
	private PrintViewer printview;

	/**
	 * Der Seitentitel, kommt aus dem Ordner Titelfeld
	 */
	final private String titel;
	/**
	 * Das Page Objekt zur Druckvorschau
	 */
	final private Print page;
	/**
	 * Die Hauptanwendnung zu dem Dialog.
	 */
	final private Shell parentShell;

	/**
	 * Der Konstruktor erstellt schon mal alles, zeigt aber nix an.
	 * 
	 * @param print
	 *            das Print-Objekt das angezeigt werden soll.
	 * @param parentShell
	 *            die Hauptanwendnung zu dem Dialog
	 * @throws ApplicationException
	 */
	public PrintViewDialog(final AbstractPrint print, final Shell parentShell)
			throws ApplicationException {

		this.titel = print.getTitle();
		this.page = print.createPrint();
		this.parentShell = parentShell;
	}

	/**
	 * &Ouml;ffent die Druckvorschau in einem Dialogfenster.
	 * 
	 * @return Die Shell f&uuml; den Dialog.
	 * @throws ApplicationException
	 */
	public Shell open() throws ApplicationException {

		shell = new Shell(parentShell, SWT.SHELL_TRIM | SWT.RESIZE
				| SWT.APPLICATION_MODAL);
		shell.setText("Print Preview");
		shell.setBounds(100, 100, 830, 630);
		shell.setLayout(new GridLayout(1, false));

		try {
			createButtonPanel(shell);
		} catch (RemoteException e) {
			throw new ApplicationException(
					I18N.tr("Error opening print preview"), e);
		}

		createPrinterView(shell);
		if (page instanceof PagePrint) {
			final PagePrint pagePrint = (PagePrint) page;
			// Ein wenig Abstand am Ende einfgen
			if (pagePrint.getBody() instanceof GridPrint) {
				final GridPrint body = (GridPrint) pagePrint.getBody();
				final int width = body.getColumns()[0].size;
				final EmptyPrint topP = new EmptyPrint(width, 20);
				body.add(topP);
			}
		}
		printview.setPrint(page);

		shell.setVisible(true);

		while (!shell.isDisposed()) {
			if (!shell.getDisplay().readAndDispatch()) {
				shell.getDisplay().sleep();
			}
		}

		return shell;
	}

	/**
	 * Erstellt die Button zum drucken, bzw. Abbrechen
	 * 
	 * @param parent
	 *            Wo die Button gemalt werden sollen
	 * @throws RemoteException
	 */
	private void createButtonPanel(final Composite parent)
			throws RemoteException {

		final Button print = new Button("Drucken", new Action() {

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {
				// Das Drucken behandeln wir in eine extra Funktion
				handlePrint();
			}
		}, null, false, "document-print.png");

		final Button cancel = new Button("Schlieen", new Action() {

			/**
			 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
			 */
			@Override
			public void handleAction(final Object context)
					throws ApplicationException {
				// Das Drucken behandeln wir in eine extra Funktion
				shell.dispose();
			}
		}, null, false, "window-close.png");

		final ButtonArea buttonArea = new ButtonArea();
		buttonArea.addButton(print);
		buttonArea.addButton(cancel);
		buttonArea.paint(parent);

	}

	/**
	 * Erstellt und liefert die eigentliche Druckvorschau
	 * 
	 * @param parent
	 *            in der die Druckvorschau gemalt werden soll
	 * @return
	 */
	private Control createPrinterView(final Composite parent) {

		scroll = new ScrolledContainer(parent, 3);

		// Oberer Rand mit Hinweistext
		final Label hint = new Label(scroll.getComposite(), SWT.NONE);
		final GridData gd1 = new GridData();
		gd1.heightHint = (int) (12 * 0.039 * 96);
		gd1.horizontalSpan = 3;
		hint.setLayoutData(gd1);
		final String hinweis = "Die Druckvorschau soll nur einen groben Eindruck vermitteln. "
				+ "Der echte Ausdruck kann deutlich abweichen.\n"
				+ "Hinweis: Ist mehr als ein Scrollbalken sichtbar, passt der "
				+ "Ausdruck wahrscheinlich nicht auf eine Seite";
		hint.setText(hinweis);

		// linker Rand
		final Label left = new Label(scroll.getComposite(), SWT.NONE);
		left.setText("");
		GridData gridData = new GridData();
		gridData.widthHint = (int) (8 * 0.039 * 96);
		left.setLayoutData(gridData);

		// Container fr den PrintViewer, damit die Gre halbwegs passt
		final Composite container = new Composite(scroll.getComposite(),
				SWT.BORDER);
		final GridLayout layout = new GridLayout(1, false);
		container.setLayout(layout);
		container.setBackground(new Color(parent.getDisplay(), 255, 255, 255));

		// rechter Rand
		final Label right = new Label(scroll.getComposite(), SWT.NONE);
		right.setText("");
		gridData = new GridData();
		gridData.widthHint = (int) (8 * 0.039 * 96);
		right.setLayoutData(gridData);
		
		
		// Linken Rand, damit es schne raussieht
		final Label linkerRand = new Label(container, SWT.NONE);
		gridData = new GridData();
		gridData.heightHint = (int) (10 * 0.039 * 96);
		linkerRand.setLayoutData(gridData);
		linkerRand.setText("");

		// Grob ein A4 Blatt mit 2 cm Rand
		gridData = new GridData();
		gridData.widthHint = (int) (160 * 0.039 * 96);
		gridData.heightHint = (int) (290 * 0.039 * 96);
		gridData.horizontalIndent = (int) (20 * 0.039 * 96);
		
		


		printview = new PrintViewer(container, SWT.NONE);
		printview.getControl().setLayoutData(gridData);

		// Eigener Listener fr den PrintViewer, da er ja selbst ein
		// ScrolledComposit ist
		printview.getControl()
				.addMouseWheelListener(new MyMouseWheelListener());
		// scroll Container an Gre anpassen
		scroll.update();

		return scroll.getComposite();

	}

	/**
	 * Hier wird gedruckt
	 */
	protected void handlePrint() {
		final PrintDialog dialog = new PrintDialog(shell, SWT.NONE);
		final PrinterData printerData = dialog.open();
		if (printerData != null) {

			// Erstelle Printjob
			final PrintJob job = new PrintJob("OrdnerVerwaltung-" + titel, page);

			final Margins margins = job.getMargins();
			margins.left = 100;
			margins.right = 50;

			// Drucken starten
			PaperClips.print(job, printerData);

			// ...und wir schlieen die Vorschau
			shell.dispose();

		}
	}

	/**
	 * Unser privater MouseWheelListener
	 * 
	 * @author munstfr
	 * @version 1.0
	 */
	@SuppressWarnings("PMD.AtLeastOneConstructor")
	private class MyMouseWheelListener implements MouseWheelListener {
		/**
		 * Die Bildschirmauflsung
		 */
		private final Point dpi = Display.getDefault().getDPI();

		/**
		 * @see org.eclipse.swt.events.MouseWheelListener#mouseScrolled(org.eclipse.swt.events.MouseEvent)
		 */
		@Override
		@SuppressWarnings("PMD.DataflowAnomalyAnalysis")
		public void mouseScrolled(final MouseEvent event) {

			final ScrolledComposite scrolledComposite = (ScrolledComposite) scroll
					.getComposite().getParent();

			final Rectangle bounds = scrolledComposite.getClientArea();
			Point origin = scrolledComposite.getOrigin();
			final int direction = event.count > 0 ? -1 : 1;
			final Point size = printview.getControl().getSize();
			if (size.y > bounds.height && event.stateMask == SWT.NONE) {
				origin.y += direction * Math.min(dpi.y, bounds.height / 4);
			} else if (size.x > bounds.width) {
				origin.x += direction * Math.min(dpi.x, bounds.width / 4);
			}
			scrolledComposite.setOrigin(origin);
		}
	}
}
